@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -34,99 +34,148 @@ import com.puppycrawl.tools.checkstyle.utils.CheckUtil;
 import com.puppycrawl.tools.checkstyle.utils.ScopeUtil;
 
 /**
+ * <p>
  * Checks that a local variable or a parameter does not shadow
  * a field that is defined in the same class.
- *
- * <p>An example of how to configure the check is:
- * <pre>
- * &lt;module name="HiddenField"/&gt;
- * </pre>
- *
- * <p>An example of how to configure the check so that it checks variables but not
- * parameters is:
- * <pre>
- * &lt;module name="HiddenField"&gt;
- *    &lt;property name="tokens" value="VARIABLE_DEF"/&gt;
- * &lt;/module&gt;
- * </pre>
- *
- * <p>An example of how to configure the check so that it ignores the parameter of
- * a setter method is:
- * <pre>
- * &lt;module name="HiddenField"&gt;
- *    &lt;property name="ignoreSetter" value="true"/&gt;
- * &lt;/module&gt;
- * </pre>
- *
- * <p>A method is recognized as a setter if it is in the following form
+ * </p>
+ * <p>
+ * It is possible to configure the check to ignore all property setter methods.
+ * </p>
+ * <p>
+ * A method is recognized as a setter if it is in the following form
+ * </p>
  * <pre>
  * ${returnType} set${Name}(${anyType} ${name}) { ... }
  * </pre>
+ * <p>
  * where ${anyType} is any primitive type, class or interface name;
  * ${name} is name of the variable that is being set and ${Name} its
  * capitalized form that appears in the method name. By default it is expected
  * that setter returns void, i.e. ${returnType} is 'void'. For example
+ * </p>
  * <pre>
  * void setTime(long time) { ... }
  * </pre>
+ * <p>
  * Any other return types will not let method match a setter pattern. However,
  * by setting <em>setterCanReturnItsClass</em> property to <em>true</em>
  * definition of a setter is expanded, so that setter return type can also be
  * a class in which setter is declared. For example
+ * </p>
  * <pre>
  * class PageBuilder {
  *   PageBuilder setName(String name) { ... }
  * }
  * </pre>
+ * <p>
  * Such methods are known as chain-setters and a common when Builder-pattern
  * is used. Property <em>setterCanReturnItsClass</em> has effect only if
  * <em>ignoreSetter</em> is set to true.
- *
- * <p>An example of how to configure the check so that it ignores the parameter
- * of either a setter that returns void or a chain-setter.
+ * </p>
+ * <ul>
+ * <li>
+ * Property {@code ignoreFormat} - Define the RegExp for names of variables
+ * and parameters to ignore.
+ * Default value is {@code null}.
+ * </li>
+ * <li>
+ * Property {@code ignoreConstructorParameter} - Control whether to ignore constructor parameters.
+ * Default value is {@code false}.
+ * </li>
+ * <li>
+ * Property {@code ignoreSetter} - Allow to ignore the parameter of a property setter method.
+ * Default value is {@code false}.
+ * </li>
+ * <li>
+ * Property {@code setterCanReturnItsClass} - Allow to expand the definition of a setter method
+ * to include methods that return the class' instance.
+ * Default value is {@code false}.
+ * </li>
+ * <li>
+ * Property {@code ignoreAbstractMethods} - Control whether to ignore parameters
+ * of abstract methods.
+ * Default value is {@code false}.
+ * </li>
+ * <li>
+ * Property {@code tokens} - tokens to check
+ * Default value is:
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#VARIABLE_DEF">
+ * VARIABLE_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#PARAMETER_DEF">
+ * PARAMETER_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#LAMBDA">
+ * LAMBDA</a>.
+ * </li>
+ * </ul>
+ * <p>
+ * To configure the check:
+ * </p>
  * <pre>
- * &lt;module name="HiddenField"&gt;
- *    &lt;property name="ignoreSetter" value="true"/&gt;
- *    &lt;property name="setterCanReturnItsClass" value="true"/&gt;
+ *  &lt;module name=&quot;HiddenField&quot;/&gt;
+ * </pre>
+ *
+ * <p>
+ * To configure the check so that it checks local variables but not parameters:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;HiddenField&quot;&gt;
+ *   &lt;property name=&quot;tokens&quot; value=&quot;VARIABLE_DEF&quot;/&gt;
  * &lt;/module&gt;
  * </pre>
  *
- * <p>An example of how to configure the check so that it ignores constructor
- * parameters is:
+ * <p>
+ * To configure the check so that it ignores the variables and parameters named "test":
+ * </p>
  * <pre>
- * &lt;module name="HiddenField"&gt;
- *    &lt;property name="ignoreConstructorParameter" value="true"/&gt;
+ * &lt;module name=&quot;HiddenField&quot;&gt;
+ *   &lt;property name=&quot;ignoreFormat&quot; value=&quot;^test$&quot;/&gt;
  * &lt;/module&gt;
  * </pre>
- *
- * <p>An example of how to configure the check so that it ignores variables and parameters
- * named 'test':
  * <pre>
- * &lt;module name="HiddenField"&gt;
- *    &lt;property name="ignoreFormat" value="^test$"/&gt;
- * &lt;/module&gt;
- * </pre>
- *
- * <pre>
- * {@code
  * class SomeClass
  * {
- *     private List&lt;String&gt; test;
+ *   private List&lt;String&gt; test;
  *
- *     private void addTest(List&lt;String&gt; test) // no violation
- *     {
- *         this.test.addAll(test);
- *     }
+ *   private void addTest(List&lt;String&gt; test) // no violation
+ *   {
+ *     this.test.addAll(test);
+ *   }
  *
- *     private void foo()
- *     {
- *         final List&lt;String&gt; test = new ArrayList&lt;&gt;(); // no violation
- *         ...
- *     }
- * }
+ *   private void foo()
+ *   {
+ *     final List&lt;String&gt; test = new ArrayList&lt;&gt;(); // no violation
+ *     ...
+ *   }
  * }
  * </pre>
+ * <p>
+ * To configure the check so that it ignores constructor parameters:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;HiddenField&quot;&gt;
+ *   &lt;property name=&quot;ignoreConstructorParameter&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <p>
+ * To configure the check so that it ignores the parameter of setter methods:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;HiddenField&quot;&gt;
+ *   &lt;property name=&quot;ignoreSetter&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <p>
+ * To configure the check so that it ignores the parameter of setter methods
+ * recognizing setter as returning either {@code void} or a class in which it is declared:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;HiddenField&quot;&gt;
+ *   &lt;property name=&quot;ignoreSetter&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;setterCanReturnItsClass&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
  *
+ * @since 3.0
  */
 @FileStatefulCheck
 public class HiddenFieldCheck
@@ -138,29 +187,30 @@ public class HiddenFieldCheck
      */
     public static final String MSG_KEY = "hidden.field";
 
-    /** Stack of sets of field names,
+    /**
+     * Stack of sets of field names,
      * one for each class of a set of nested classes.
      */
     private FieldFrame frame;
 
-    /** Pattern for names of variables and parameters to ignore. */
+    /** Define the RegExp for names of variables and parameters to ignore. */
     private Pattern ignoreFormat;
 
-    /** Controls whether to check the parameter of a property setter method. */
+    /**
+     * Allow to ignore the parameter of a property setter method.
+     */
     private boolean ignoreSetter;
 
     /**
-     * If ignoreSetter is set to true then this variable controls what
-     * the setter method can return By default setter must return void.
-     * However, is this variable is set to true then setter can also
-     * return class in which is declared.
+     * Allow to expand the definition of a setter method to include methods
+     * that return the class' instance.
      */
     private boolean setterCanReturnItsClass;
 
-    /** Controls whether to check the parameter of a constructor. */
+    /** Control whether to ignore constructor parameters. */
     private boolean ignoreConstructorParameter;
 
-    /** Controls whether to check the parameter of abstract methods. */
+    /** Control whether to ignore parameters of abstract methods. */
     private boolean ignoreAbstractMethods;
 
     @Override
@@ -226,10 +276,6 @@ public class HiddenFieldCheck
                 log(firstChild, MSG_KEY, untypedLambdaParameterName);
             }
         }
-        else {
-            // Type of lambda parameter is not omitted.
-            processVariable(ast);
-        }
     }
 
     /**
@@ -501,7 +547,7 @@ public class HiddenFieldCheck
     }
 
     /**
-     * Set the ignore format for the specified regular expression.
+     * Setter to define the RegExp for names of variables and parameters to ignore.
      * @param pattern a pattern.
      */
     public void setIgnoreFormat(Pattern pattern) {
@@ -509,7 +555,7 @@ public class HiddenFieldCheck
     }
 
     /**
-     * Set whether to ignore the parameter of a property setter method.
+     * Setter to allow to ignore the parameter of a property setter method.
      * @param ignoreSetter decide whether to ignore the parameter of
      *     a property setter method.
      */
@@ -518,8 +564,8 @@ public class HiddenFieldCheck
     }
 
     /**
-     * Controls if setter can return only void (default behavior) or it
-     * can also return class in which it is declared.
+     * Setter to allow to expand the definition of a setter method to include methods
+     * that return the class' instance.
      *
      * @param aSetterCanReturnItsClass if true then setter can return
      *        either void or class in which it is declared. If false then
@@ -533,7 +579,7 @@ public class HiddenFieldCheck
     }
 
     /**
-     * Set whether to ignore constructor parameters.
+     * Setter to control whether to ignore constructor parameters.
      * @param ignoreConstructorParameter decide whether to ignore
      *     constructor parameters.
      */
@@ -543,7 +589,7 @@ public class HiddenFieldCheck
     }
 
     /**
-     * Set whether to ignore parameters of abstract methods.
+     * Setter to control whether to ignore parameters of abstract methods.
      * @param ignoreAbstractMethods decide whether to ignore
      *     parameters of abstract methods.
      */
@@ -578,7 +624,7 @@ public class HiddenFieldCheck
          * @param staticType is this a static inner type (class or enum).
          * @param frameName name associated with the frame, which can be a
          */
-        FieldFrame(FieldFrame parent, boolean staticType, String frameName) {
+        /* package */ FieldFrame(FieldFrame parent, boolean staticType, String frameName) {
             this.parent = parent;
             this.staticType = staticType;
             this.frameName = frameName;
