@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -31,163 +31,342 @@ import com.puppycrawl.tools.checkstyle.api.TokenTypes;
 import com.puppycrawl.tools.checkstyle.utils.CommonUtil;
 
 /**
+ * <p>
  * Checks the ordering/grouping of imports. Features are:
- * <ul>
- * <li>groups imports: ensures that groups of imports come in a specific order
- * (e.g., java. comes first, javax. comes second, then everything else)</li>
- * <li>adds a separation between groups : ensures that a blank line sit between
- * each group</li>
- * <li>import groups aren't separated internally: ensures that
- * each group aren't separated internally by blank line or comment</li>
- * <li>sorts imports inside each group: ensures that imports within each group
- * are in lexicographic order</li>
- * <li>sorts according to case: ensures that the comparison between import is
- * case sensitive</li>
- * <li>groups static imports: ensures that static imports are at the top (or the
- * bottom) of all the imports, or above (or under) each group, or are treated
- * like non static imports (@see {@link ImportOrderOption}</li>
- * </ul>.
- *
- * <pre>
- * Properties:
- * </pre>
- * <table summary="Properties" border="1">
- *   <tr><th>name</th><th>Description</th><th>type</th><th>default value</th></tr>
- *   <tr><td>option</td><td>policy on the relative order between regular imports and static
- *       imports</td><td>{@link ImportOrderOption}</td><td>under</td></tr>
- *   <tr><td>groups</td><td>list of type import groups (every group identified either by a
- *       common prefix string, or by a regular expression enclosed in forward slashes
- *       (e.g. /regexp/). All type imports, which does not match any group,
- *       falls into an additional group, located at the end. Thus, the empty list of type groups
- *       (the default value) means one group for all type imports</td>
- *       <td>list of strings</td><td>empty list</td></tr>
- *   <tr><td>ordered</td><td>whether type imports within group should be sorted</td>
- *       <td>Boolean</td><td>true</td></tr>
- *   <tr><td>separated</td><td>whether type imports groups should be separated by, at least,
- *       one blank line or comment and aren't separated internally
- *       </td><td>Boolean</td><td>false</td></tr>
- *   <tr><td>separatedStaticGroups</td><td>whether static imports should be separated by, at least,
- *       one blank line or comment and aren't separated internally
- *       </td><td>Boolean</td><td>false</td></tr>
- *   <tr><td>caseSensitive</td><td>whether string comparison should be case sensitive or not.
- *       Case sensitive sorting is in ASCII sort order</td><td>Boolean</td><td>true</td></tr>
- *   <tr><td>staticGroups</td><td>list of static import groups (every group identified either by a
- *       common prefix string, or by a regular expression enclosed in forward slashes
- *       (e.g. /regexp/). All static imports, which does not match any group,
- *       falls into an additional group, located at the end. Thus, the empty list of static groups
- *       (the default value) means one group for all static imports</td>
- *       <td>list of strings</td><td>empty list</td></tr>
- *   <tr><td>sortStaticImportsAlphabetically</td><td>whether static imports located at top or
- *       bottom are sorted within the group.</td><td>Boolean</td><td>false</td></tr>
- *   <tr><td>useContainerOrderingForStatic</td><td>whether to use container ordering
- *       (Eclipse IDE term) for static imports or not</td><td>Boolean</td><td>false</td></tr>
- * </table>
- *
- * <p>
- * Example:
  * </p>
- * <p>To configure the check so that it matches default Eclipse formatter configuration
- *    (tested on Kepler, Luna and Mars):</p>
  * <ul>
- *     <li>group of static imports is on the top</li>
- *     <li>groups of non-static imports: &quot;java&quot; then &quot;javax&quot;
- *         packages first, then &quot;org&quot; and then all other imports</li>
- *     <li>imports will be sorted in the groups</li>
- *     <li>groups are separated by, at least, one blank line and aren't separated internally</li>
+ * <li>
+ * groups type/static imports: ensures that groups of imports come in a specific order
+ * (e.g., java. comes first, javax. comes second, then everything else)
+ * </li>
+ * <li>
+ * adds a separation between type import groups : ensures that a blank line sit between each group
+ * </li>
+ * <li>
+ * type/static import groups aren't separated internally: ensures that each group aren't separated
+ * internally by blank line or comment
+ * </li>
+ * <li>
+ * sorts type/static imports inside each group: ensures that imports within each group are in
+ * lexicographic order
+ * </li>
+ * <li>
+ * sorts according to case: ensures that the comparison between imports is case sensitive, in
+ * <a href="https://en.wikipedia.org/wiki/ASCII#Order">ASCII sort order</a>
+ * </li>
+ * <li>
+ * arrange static imports: ensures the relative order between type imports and static imports
+ * (see <a href="https://checkstyle.org/property_types.html#importOrder">import orders</a>)
+ * </li>
+ * </ul>
+ * <ul>
+ * <li>
+ * Property {@code option} - specify policy on the relative order between type imports and static
+ * imports.
+ * Default value is {@code under}.
+ * </li>
+ * <li>
+ * Property {@code groups} - specify list of <b>type import</b> groups (every group identified
+ * either by a common prefix string, or by a regular expression enclosed in forward slashes
+ * (e.g. {@code /regexp/}). All type imports, which does not match any group, falls into an
+ * additional group, located at the end.
+ * Thus, the empty list of type groups (the default value) means one group for all type imports.
+ * Default value is {@code {}}.
+ * </li>
+ * <li>
+ * Property {@code ordered} - control whether type imports within each group should be
+ * sorted.
+ * It doesn't affect sorting for static imports.
+ * Default value is true.
+ * </li>
+ * <li>
+ * Property {@code separated} - control whether type import groups should be separated
+ * by, at least, one blank line or comment and aren't separated internally.
+ * It doesn't affect separations for static imports.
+ * Default value is false.
+ * </li>
+ * <li>
+ * Property {@code separatedStaticGroups} - control whether static import groups should
+ * be separated by, at least, one blank line or comment and aren't separated internally.
+ * This property has effect only when the property {@code option} is is set to {@code top}
+ * or {@code bottom}.
+ * Default value is false.
+ * </li>
+ * <li>
+ * Property {@code caseSensitive} - control whether string comparison should be case
+ * sensitive or not. Case sensitive sorting is in
+ * <a href="https://en.wikipedia.org/wiki/ASCII#Order">ASCII sort order</a>.
+ * It affects both type imports and static imports.
+ * Default value is true.
+ * </li>
+ * <li>
+ * Property {@code staticGroups} - specify list of <b>static</b> import groups (every group
+ * identified either by a common prefix string, or by a regular expression enclosed in forward
+ * slashes (e.g. {@code /regexp/}). All static imports, which does not match any group, falls into
+ * an additional group, located at the end. Thus, the empty list of static groups (the default
+ * value) means one group for all static imports. This property has effect only when the property
+ * {@code option} is set to {@code top} or {@code bottom}.
+ * Default value is {@code {}}.
+ * </li>
+ * <li>
+ * Property {@code sortStaticImportsAlphabetically} - control whether
+ * <b>static imports</b> located at <b>top</b> or <b>bottom</b> are sorted within the group.
+ * Default value is false.
+ * </li>
+ * <li>
+ * Property {@code useContainerOrderingForStatic} - control whether to use container
+ * ordering (Eclipse IDE term) for static imports or not.
+ * Default value is false.
+ * </li>
+ * <li>
+ * Property {@code tokens} - tokens to check
+ * Default value is:
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#STATIC_IMPORT">
+ * STATIC_IMPORT</a>.
+ * </li>
+ * </ul>
+ * <p>
+ * To configure the check so that it matches default Eclipse formatter configuration
+ * (tested on Kepler and Luna releases):
+ * </p>
+ * <ul>
+ * <li>
+ * group of static imports is on the top
+ * </li>
+ * <li>
+ * groups of type imports: "java" and "javax" packages first, then "org" and then all other imports
+ * </li>
+ * <li>
+ * imports will be sorted in the groups
+ * </li>
+ * <li>
+ * groups are separated by, at least, one blank line and aren't separated internally
+ * </li>
+ * </ul>
+ * <p>
+ * Notes:
+ * </p>
+ * <ul>
+ * <li>
+ * "com" package is not mentioned on configuration, because it is ignored by Eclipse Kepler and Luna
+ * (looks like Eclipse defect)
+ * </li>
+ * <li>
+ * configuration below doesn't work in all 100% cases due to inconsistent behavior prior to
+ * Mars release, but covers most scenarios
+ * </li>
  * </ul>
- *
  * <pre>
  * &lt;module name=&quot;ImportOrder&quot;&gt;
- *    &lt;property name=&quot;groups&quot; value=&quot;/^javax?\./,org&quot;/&gt;
- *    &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
- *    &lt;property name=&quot;separated&quot; value=&quot;true&quot;/&gt;
- *    &lt;property name=&quot;option&quot; value=&quot;above&quot;/&gt;
- *    &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;groups&quot; value=&quot;/^java\./,javax,org&quot;/&gt;
+ *   &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;separated&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;above&quot;/&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
  * &lt;/module&gt;
  * </pre>
- *
- * <p>To configure the check so that it matches default IntelliJ IDEA formatter configuration
- *    (tested on v14):</p>
+ * <p>
+ * To configure the check so that it matches default Eclipse formatter configuration
+ * (tested on Mars release):
+ * </p>
  * <ul>
- *     <li>group of static imports is on the bottom</li>
- *     <li>groups of non-static imports: all imports except of &quot;javax&quot; and
- *         &quot;java&quot;, then &quot;javax&quot; and &quot;java&quot;</li>
- *     <li>imports will be sorted in the groups</li>
- *     <li>groups are separated by, at least, one blank line and aren't separated internally</li>
+ * <li>
+ * group of static imports is on the top
+ * </li>
+ * <li>
+ * groups of type imports: "java" and "javax" packages first, then "org" and "com",
+ * then all other imports as one group
+ * </li>
+ * <li>
+ * imports will be sorted in the groups
+ * </li>
+ * <li>
+ * groups are separated by, at least, one blank line and aren't separated internally
+ * </li>
  * </ul>
- *
- *         <p>
- *         Note: &quot;separated&quot; option is disabled because IDEA default has blank line
- *         between &quot;java&quot; and static imports, and no blank line between
- *         &quot;javax&quot; and &quot;java&quot;
- *         </p>
- *
  * <pre>
  * &lt;module name=&quot;ImportOrder&quot;&gt;
- *     &lt;property name=&quot;groups&quot; value=&quot;*,javax,java&quot;/&gt;
- *     &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
- *     &lt;property name=&quot;separated&quot; value=&quot;false&quot;/&gt;
- *     &lt;property name=&quot;option&quot; value=&quot;bottom&quot;/&gt;
- *     &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;groups&quot; value=&quot;/^java\./,javax,org,com&quot;/&gt;
+ *   &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;separated&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;above&quot;/&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
  * &lt;/module&gt;
  * </pre>
- *
- * <p>To configure the check so that it matches default NetBeans formatter configuration
- *    (tested on v8):</p>
+ * <p>
+ * To configure the check so that it matches default IntelliJ IDEA formatter configuration
+ * (tested on v2018.2):
+ * </p>
  * <ul>
- *     <li>groups of non-static imports are not defined, all imports will be sorted
- *         as a one group</li>
- *     <li>static imports are not separated, they will be sorted along with other imports</li>
+ * <li>
+ * group of static imports is on the bottom
+ * </li>
+ * <li>
+ * groups of type imports: all imports except of "javax" and "java", then "javax" and "java"
+ * </li>
+ * <li>
+ * imports will be sorted in the groups
+ * </li>
+ * <li>
+ * groups are separated by, at least, one blank line and aren't separated internally
+ * </li>
  * </ul>
- *
+ * <p>
+ * Note: a <a href="https://checkstyle.org/config_filters.html#SuppressionXpathSingleFilter">
+ * suppression xpath single filter</a> is needed because
+ * IDEA has no blank line between "javax" and "java".
+ * ImportOrder has a limitation by design to enforce an empty line between groups ("java", "javax").
+ * There is no flexibility to enforce empty lines between some groups and no empty lines between
+ * other groups.
+ * </p>
+ * <p>
+ * Note: "separated" option is disabled because IDEA default has blank line between "java" and
+ * static imports, and no blank line between "javax" and "java".
+ * </p>
  * <pre>
  * &lt;module name=&quot;ImportOrder&quot;&gt;
- *     &lt;property name=&quot;option&quot; value=&quot;inflow&quot;/&gt;
+ *   &lt;property name=&quot;groups&quot; value=&quot;*,javax,java&quot;/&gt;
+ *   &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;separated&quot; value=&quot;false&quot;/&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;bottom&quot;/&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * &lt;module name="SuppressionXpathSingleFilter"&gt;
+ *   &lt;property name="checks" value="ImportOrder"/&gt;
+ *   &lt;property name="message" value="^'java\..*'.*"/&gt;
  * &lt;/module&gt;
  * </pre>
- *
  * <p>
- * Group descriptions enclosed in slashes are interpreted as regular
- * expressions. If multiple groups match, the one matching a longer
- * substring of the imported name will take precedence, with ties
- * broken first in favor of earlier matches and finally in favor of
- * the first matching group.
+ * To configure the check so that it matches default NetBeans formatter configuration
+ * (tested on v8):
  * </p>
- *
- * <p>
- * There is always a wildcard group to which everything not in a named group
- * belongs. If an import does not match a named group, the group belongs to
- * this wildcard group. The wildcard group position can be specified using the
- * {@code *} character.
- * </p>
- *
- * <p>Check also has on option making it more flexible:
- * <b>sortStaticImportsAlphabetically</b> - sets whether static imports grouped by
- * <b>top</b> or <b>bottom</b> option should be sorted alphabetically or
- * not, default value is <b>false</b>. It is applied to static imports grouped
- * with <b>top</b> or <b>bottom</b> options.<br>
- * This option is helping in reconciling of this Check and other tools like
- * Eclipse's Organize Imports feature.
- * </p>
- * <p>
- * To configure the Check allows static imports grouped to the <b>top</b>
- * being sorted alphabetically:
- * </p>
- *
+ * <ul>
+ * <li>
+ * groups of type imports are not defined, all imports will be sorted as a one group
+ * </li>
+ * <li>
+ * static imports are not separated, they will be sorted along with other imports
+ * </li>
+ * </ul>
  * <pre>
- * {@code
- * import static java.lang.Math.abs;
+ * &lt;module name=&quot;ImportOrder&quot;&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;inflow&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <p>
+ * Group descriptions enclosed in slashes are interpreted as regular expressions.
+ * If multiple groups match, the one matching a longer substring of the imported name
+ * will take precedence, with ties broken first in favor of earlier matches and finally
+ * in favor of the first matching group.
+ * </p>
+ * <p>
+ * There is always a wildcard group to which everything not in a named group belongs.
+ * If an import does not match a named group, the group belongs to this wildcard group.
+ * The wildcard group position can be specified using the {@code *} character.
+ * </p>
+ * <p>
+ * Check also has on option making it more flexible: <b>sortStaticImportsAlphabetically</b>
+ * - sets whether static imports grouped by <b>top</b> or <b>bottom</b> option should be sorted
+ * alphabetically or not, default value is <b>false</b>. It is applied to static imports grouped
+ * with <b>top</b> or <b>bottom</b> options. This option is helping in reconciling of this
+ * Check and other tools like Eclipse's Organize Imports feature.
+ * </p>
+ * <p>
+ * To configure the Check allows static imports grouped to the <b>top</b> being sorted
+ * alphabetically:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;ImportOrder&quot;&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;top&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * import static java.lang.Math.PI;
+ * import static java.lang.Math.abs; // OK, alphabetical case sensitive ASCII order, 'P' &lt; 'a'
  * import static org.abego.treelayout.Configuration.AlignmentInLevel; // OK, alphabetical order
  *
  * import org.abego.*;
  *
- * import java.util.Set;
+ * import java.util.Set; //  Wrong order for 'java.util.Set' import.
  *
  * public class SomeClass { ... }
- * }
+ * </pre>
+ * <p>
+ * To configure the Check with groups of static imports:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;ImportOrder&quot;&gt;
+ *   &lt;property name=&quot;staticGroups&quot; value=&quot;org,java&quot;/&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * import static org.abego.treelayout.Configuration.AlignmentInLevel; // Group 1
+ * import static java.lang.Math.abs; // Group 2
+ * import static java.lang.String.format; // Group 2
+ * import static com.google.common.primitives.Doubles.BYTES; // Group "everything else"
+ *
+ * public class SomeClass { ... }
+ * </pre>
+ * <p>
+ * The following example shows the idea of 'useContainerOrderingForStatic' option that is
+ * useful for Eclipse IDE users to match ordering validation.
+ * This is how the import comparison works for static imports: we first compare
+ * the container of the static import, container is the type enclosing the static element
+ * being imported. When the result of the comparison is 0 (containers are equal),
+ * we compare the fully qualified import names.
+ * For e.g. this is what is considered to be container names for the given example:
+ *
+ * import static HttpConstants.COLON     =&gt; HttpConstants
+ * import static HttpHeaders.addHeader   =&gt; HttpHeaders
+ * import static HttpHeaders.setHeader   =&gt; HttpHeaders
+ * import static HttpHeaders.Names.DATE  =&gt; HttpHeaders.Names
+ *
+ * According to this logic, HttpHeaders.Names should come after HttpHeaders.
+ * </p>
+ * <p>
+ * Example for {@code useContainerOrderingForStatic=true}
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;ImportOrder&quot;&gt;
+ *   &lt;property name=&quot;useContainerOrderingForStatic&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;top&quot;/&gt;
+ *   &lt;property name=&quot;caseSensitive&quot; value=&quot;false&quot;/&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * import static io.netty.handler.codec.http.HttpConstants.COLON;
+ * import static io.netty.handler.codec.http.HttpHeaders.addHeader;
+ * import static io.netty.handler.codec.http.HttpHeaders.setHeader;
+ * import static io.netty.handler.codec.http.HttpHeaders.Names.DATE;
+ *
+ * public class InputEclipseStaticImportsOrder { }
+ * </pre>
+ * <p>
+ * Example for {@code useContainerOrderingForStatic=false}
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;ImportOrder&quot;&gt;
+ *   &lt;property name=&quot;useContainerOrderingForStatic&quot; value=&quot;false&quot;/&gt;
+ *   &lt;property name=&quot;ordered&quot; value=&quot;true&quot;/&gt;
+ *   &lt;property name=&quot;option&quot; value=&quot;top&quot;/&gt;
+ *   &lt;property name=&quot;caseSensitive&quot; value=&quot;false&quot;/&gt;
+ *   &lt;property name=&quot;sortStaticImportsAlphabetically&quot; value=&quot;true&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * import static io.netty.handler.codec.http.HttpConstants.COLON;
+ * import static io.netty.handler.codec.http.HttpHeaders.addHeader;
+ * import static io.netty.handler.codec.http.HttpHeaders.setHeader;
+ * import static io.netty.handler.codec.http.HttpHeaders.Names.DATE; // violation
+ *
+ * public class InputEclipseStaticImportsOrder { }
  * </pre>
  *
- *
+ * @since 3.2
  */
 @FileStatefulCheck
 public class ImportOrderCheck
@@ -217,17 +396,50 @@ public class ImportOrderCheck
     /** Empty array of pattern type needed to initialize check. */
     private static final Pattern[] EMPTY_PATTERN_ARRAY = new Pattern[0];
 
-    /** List of type import groups specified by the user. */
+    /**
+     * Specify list of <b>type import</b> groups (every group identified either by a common prefix
+     * string, or by a regular expression enclosed in forward slashes (e.g. {@code /regexp/}).
+     * All type imports, which does not match any group, falls into an additional group,
+     * located at the end. Thus, the empty list of type groups (the default value) means one group
+     * for all type imports.
+     */
     private Pattern[] groups = EMPTY_PATTERN_ARRAY;
-    /** List of static import groups specified by the user. */
+
+    /**
+     * Specify list of <b>static</b> import groups (every group identified either by a common prefix
+     * string, or by a regular expression enclosed in forward slashes (e.g. {@code /regexp/}).
+     * All static imports, which does not match any group, falls into an additional group, located
+     * at the end. Thus, the empty list of static groups (the default value) means one group for all
+     * static imports. This property has effect only when the property {@code option} is set to
+     * {@code top} or {@code bottom}.
+     */
     private Pattern[] staticGroups = EMPTY_PATTERN_ARRAY;
-    /** Require imports in group be separated. */
+
+    /**
+     * Control whether type import groups should be separated by, at least, one blank
+     * line or comment and aren't separated internally. It doesn't affect separations for static
+     * imports.
+     */
     private boolean separated;
-    /** Require static imports in group be separated. */
+
+    /**
+     * Control whether static import groups should be separated by, at least, one blank
+     * line or comment and aren't separated internally. This property has effect only when the
+     * property {@code option} is is set to {@code top} or {@code bottom}.
+     */
     private boolean separatedStaticGroups;
-    /** Require imports in group. */
+
+    /**
+     * Control whether type imports within each group should be sorted.
+     * It doesn't affect sorting for static imports.
+     */
     private boolean ordered = true;
-    /** Should comparison be case sensitive. */
+
+    /**
+     * Control whether string comparison should be case sensitive or not. Case sensitive
+     * sorting is in <a href="https://en.wikipedia.org/wiki/ASCII#Order">ASCII sort order</a>.
+     * It affects both type imports and static imports.
+     */
     private boolean caseSensitive = true;
 
     /** Last imported group. */
@@ -240,38 +452,47 @@ public class ImportOrderCheck
     private boolean lastImportStatic;
     /** Whether there was any imports. */
     private boolean beforeFirstImport;
-    /** Whether static and type import groups should be split apart.
+    /**
+     * Whether static and type import groups should be split apart.
      * When the {@code option} property is set to {@code INFLOW}, {@code BELOW} or {@code UNDER},
      * both the type and static imports use the properties {@code groups} and {@code separated}.
      * When the {@code option} property is set to {@code TOP} or {@code BOTTOM}, static imports
      * uses the properties {@code staticGroups} and {@code separatedStaticGroups}.
      **/
     private boolean staticImportsApart;
-    /** Whether static imports should be sorted alphabetically or not. */
+
+    /**
+     * Control whether <b>static imports</b> located at <b>top</b> or <b>bottom</b> are
+     * sorted within the group.
+     */
     private boolean sortStaticImportsAlphabetically;
-    /** Whether to use container ordering (Eclipse IDE term) for static imports or not. */
+
+    /**
+     * Control whether to use container ordering (Eclipse IDE term) for static imports
+     * or not.
+     */
     private boolean useContainerOrderingForStatic;
 
-    /** The policy to enforce. */
+    /**
+     * Specify policy on the relative order between type imports and static imports.
+     */
     private ImportOrderOption option = ImportOrderOption.UNDER;
 
     /**
-     * Set the option to enforce.
+     * Setter to specify policy on the relative order between type imports and static imports.
      * @param optionStr string to decode option from
      * @throws IllegalArgumentException if unable to decode
      */
     public void setOption(String optionStr) {
-        try {
-            option = ImportOrderOption.valueOf(optionStr.trim().toUpperCase(Locale.ENGLISH));
-        }
-        catch (IllegalArgumentException iae) {
-            throw new IllegalArgumentException("unable to parse " + optionStr, iae);
-        }
+        option = ImportOrderOption.valueOf(optionStr.trim().toUpperCase(Locale.ENGLISH));
     }
 
     /**
-     * Sets the list of package groups for type imports and the order they should occur in the
-     * file.
+     * Setter to specify list of <b>type import</b> groups (every group identified either by a
+     * common prefix string, or by a regular expression enclosed in forward slashes
+     * (e.g. {@code /regexp/}). All type imports, which does not match any group, falls into an
+     * additional group, located at the end. Thus, the empty list of type groups (the default value)
+     * means one group for all type imports.
      *
      * @param packageGroups a comma-separated list of package names/prefixes.
      */
@@ -280,9 +501,12 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets the list of package groups for static imports and the order they should occur in the
-     * file. This property has effect only when the property {@code option} is set to {@code top}
-     * or {@code bottom}.)
+     * Setter to specify list of <b>static</b> import groups (every group identified either by a
+     * common prefix string, or by a regular expression enclosed in forward slashes
+     * (e.g. {@code /regexp/}). All static imports, which does not match any group, falls into an
+     * additional group, located at the end. Thus, the empty list of static groups (the default
+     * value) means one group for all static imports. This property has effect only when
+     * the property {@code option} is set to {@code top} or {@code bottom}.
      *
      * @param packageGroups a comma-separated list of package names/prefixes.
      */
@@ -291,8 +515,8 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets whether or not imports should be ordered within any one group of
-     * imports.
+     * Setter to control whether type imports within each group should be sorted.
+     * It doesn't affect sorting for static imports.
      *
      * @param ordered
      *            whether lexicographic ordering of imports within a group
@@ -303,8 +527,9 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets whether or not groups of type imports must be separated from one another
-     * by at least one blank line or comment.
+     * Setter to control whether type import groups should be separated by, at least,
+     * one blank line or comment and aren't separated internally.
+     * It doesn't affect separations for static imports.
      *
      * @param separated
      *            whether groups should be separated by one blank line or comment.
@@ -314,9 +539,10 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets whether or not groups of static imports must be separated from one another
-     * by at least one blank line or comment. This property has effect only when the property
-     * {@code option} is set to {@code top} or {@code bottom}.)
+     * Setter to control whether static import groups should be separated by, at least,
+     * one blank line or comment and aren't separated internally.
+     * This property has effect only when the property
+     * {@code option} is is set to {@code top} or {@code bottom}.
      *
      * @param separatedStaticGroups
      *            whether groups should be separated by one blank line or comment.
@@ -326,8 +552,10 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets whether string comparison should be case sensitive or not.
-     *
+     * Setter to control whether string comparison should be case sensitive or not.
+     * Case sensitive sorting is in
+     * <a href="https://en.wikipedia.org/wiki/ASCII#Order">ASCII sort order</a>.
+     * It affects both type imports and static imports.
      * @param caseSensitive
      *            whether string comparison should be case sensitive.
      */
@@ -336,8 +564,8 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets whether static imports (when grouped using 'top' and 'bottom' option)
-     * are sorted alphabetically or according to the package groupings.
+     * Setter to control whether <b>static imports</b> located at <b>top</b> or
+     * <b>bottom</b> are sorted within the group.
      * @param sortAlphabetically true or false.
      */
     public void setSortStaticImportsAlphabetically(boolean sortAlphabetically) {
@@ -345,7 +573,8 @@ public class ImportOrderCheck
     }
 
     /**
-     * Sets whether to use container ordering (Eclipse IDE term) for static imports or not.
+     * Setter to control whether to use container ordering (Eclipse IDE term) for static
+     * imports or not.
      * @param useContainerOrdering whether to use container ordering for static imports or not.
      */
     public void setUseContainerOrderingForStatic(boolean useContainerOrdering) {
@@ -538,19 +767,19 @@ public class ImportOrderCheck
         final boolean result;
         if (isStatic) {
             if (useContainerOrderingForStatic) {
-                result = compareContainerOrder(lastImport, name, caseSensitive) >= 0;
+                result = compareContainerOrder(lastImport, name, caseSensitive) > 0;
             }
             else if (staticImportsApart) {
                 result = sortStaticImportsAlphabetically
-                    && compare(lastImport, name, caseSensitive) >= 0;
+                    && compare(lastImport, name, caseSensitive) > 0;
             }
             else {
-                result = compare(lastImport, name, caseSensitive) >= 0;
+                result = compare(lastImport, name, caseSensitive) > 0;
             }
         }
         else {
             // out of lexicographic order
-            result = compare(lastImport, name, caseSensitive) >= 0;
+            result = compare(lastImport, name, caseSensitive) > 0;
         }
         return result;
     }
@@ -727,7 +956,7 @@ public class ImportOrderCheck
             }
             else if (CommonUtil.startsWithChar(pkg, '/')) {
                 if (!CommonUtil.endsWithChar(pkg, '/')) {
-                    throw new IllegalArgumentException("Invalid group");
+                    throw new IllegalArgumentException("Invalid group: " + pkg);
                 }
                 pkg = pkg.substring(1, pkg.length() - 1);
                 grp = Pattern.compile(pkg);
