@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,105 @@ import com.puppycrawl.tools.checkstyle.api.DetailAST;
 import com.puppycrawl.tools.checkstyle.api.TokenTypes;
 
 /**
- * Checks the npath complexity against a specified limit (default = 200).
- * The npath metric computes the number of possible execution paths
- * through a function. Similar to the cyclomatic complexity but also
- * takes into account the nesting of conditional statements and
- * multi-part boolean expressions.
+ * <p>
+ * Checks the NPATH complexity against a specified limit.
+ * </p>
+ * <p>
+ * The NPATH metric computes the number of possible execution paths through a
+ * function(method). It takes into account the nesting of conditional statements
+ * and multi-part boolean expressions (A &amp;&amp; B, C || D, E ? F :G and
+ * their combinations).
+ * </p>
+ * <p>
+ * The NPATH metric was designed base on Cyclomatic complexity to avoid problem
+ * of Cyclomatic complexity metric like nesting level within a function(method).
+ * </p>
+ * <p>
+ * Metric was described at <a href="http://dl.acm.org/citation.cfm?id=42379">
+ * "NPATH: a measure of execution pathcomplexity and its applications"</a>.
+ * If you need detailed description of algorithm, please read that article,
+ * it is well written and have number of examples and details.
+ * </p>
+ * <p>
+ * Here is some quotes:
+ * </p>
+ * <blockquote>
+ * An NPATH threshold value of 200 has been established for a function.
+ * The value 200 is based on studies done at AT&amp;T Bell Laboratories [1988 year].
+ * </blockquote>
+ * <blockquote>
+ * Some of the most effective methods of reducing the NPATH value include:
+ * <ul>
+ * <li>
+ * distributing functionality;
+ * </li>
+ * <li>
+ * implementing multiple if statements as a switch statement;
+ * </li>
+ * <li>
+ * creating a separate function for logical expressions with a high count of
+ * variables and (&amp;&amp;) and or (||) operators.
+ * </li>
+ * </ul>
+ * </blockquote>
+ * <blockquote>
+ * Although strategies to reduce the NPATH complexity of functions are important,
+ * care must be taken not to distort the logical clarity of the software by
+ * applying a strategy to reduce the complexity of functions. That is, there is
+ * a point of diminishing return beyond which a further attempt at reduction of
+ * complexity distorts the logical clarity of the system structure.
+ * </blockquote>
+ * <table>
+ * <caption>Examples</caption>
+ * <thead><tr><th>Structure</th><th>Complexity expression</th></tr></thead>
+ * <tr><td>if ([expr]) { [if-range] }</td><td>NP(if-range) + 1 + NP(expr)</td></tr>
+ * <tr><td>if ([expr]) { [if-range] } else { [else-range] }</td>
+ * <td>NP(if-range)+ NP(else-range) + NP(expr)</td></tr>
+ * <tr><td>while ([expr]) { [while-range] }</td><td>NP(while-range) + NP(expr) + 1</td></tr>
+ * <tr><td>do { [do-range] } while ([expr])</td><td>NP(do-range) + NP(expr) + 1</td></tr>
+ * <tr><td>for([expr1]; [expr2]; [expr3]) { [for-range] }</td>
+ * <td>NP(for-range) + NP(expr1)+ NP(expr2) + NP(expr3) + 1</td></tr>
+ * <tr><td>switch ([expr]) { case : [case-range] default: [default-range] }</td>
+ * <td>S(i=1:i=n)NP(case-range[i]) + NP(default-range) + NP(expr)</td></tr>
+ * <tr><td>[expr1] ? [expr2] : [expr3]</td><td>NP(expr1) + NP(expr2) + NP(expr3) + 2</td></tr>
+ * <tr><td>goto label</td><td>1</td></tr><tr><td>break</td><td>1</td></tr>
+ * <tr><td>Expressions</td>
+ * <td>Number of &amp;&amp; and || operators in expression. No operators - 0</td></tr>
+ * <tr><td>continue</td><td>1</td></tr><tr><td>return</td><td>1</td></tr>
+ * <tr><td>Statement (even sequential statements)</td><td>1</td></tr>
+ * <tr><td>Empty block {}</td><td>1</td></tr><tr><td>Function call</td><td>1</td>
+ * </tr><tr><td>Function(Method) declaration or Block</td><td>P(i=1:i=N)NP(Statement[i])</td></tr>
+ * </table>
+ * <p>
+ * <b>Rationale:</b> Nejmeh says that his group had an informal NPATH limit of
+ * 200 on individual routines; functions(methods) that exceeded this value were
+ * candidates for further decomposition - or at least a closer look.
+ * <b>Please do not be fanatic with limit 200</b> - choose number that suites
+ * your project style. Limit 200 is empirical number base on some sources of at
+ * AT&amp;T Bell Laboratories of 1988 year.
+ * </p>
+ * <ul>
+ * <li>
+ * Property {@code max} - Specify the maximum threshold allowed.
+ * Default value is {@code 200}.
+ * </li>
+ * </ul>
+ * <p>
+ * To configure the check:
+ * </p>
+ * <pre>
+ * &lt;module name="NPathComplexity"/&gt;
+ * </pre>
+ * <p>
+ * To configure the check with a threshold of 1000:
+ * </p>
+ * <pre>
+ * &lt;module name="NPathComplexity"&gt;
+ *   &lt;property name="max" value="1000"/&gt;
+ * &lt;/module&gt;
+ * </pre>
  *
+ * @since 3.4
  */
 // -@cs[AbbreviationAsWordInName] Can't change check name
 @FileStatefulCheck
@@ -61,7 +154,7 @@ public final class NPathComplexityCheck extends AbstractCheck {
     private final Deque<Integer> expressionValues = new ArrayDeque<>();
 
     /** Stack of belongs to range values for question operator. */
-    private final Deque<Boolean> isAfterValues = new ArrayDeque<>();
+    private final Deque<Boolean> afterValues = new ArrayDeque<>();
 
     /**
      * Range of the last processed expression. Used for checking that ternary operation
@@ -72,14 +165,15 @@ public final class NPathComplexityCheck extends AbstractCheck {
     /** NP value for current range. */
     private BigInteger currentRangeValue = INITIAL_VALUE;
 
-    /** Threshold to report error for. */
+    /** Specify the maximum threshold allowed. */
     private int max = DEFAULT_MAX;
 
     /** True, when branch is visited, but not leaved. */
     private boolean branchVisited;
 
     /**
-     * Set the maximum threshold allowed.
+     * Setter to specify the maximum threshold allowed.
+     *
      * @param max the maximum threshold
      */
     public void setMax(int max) {
@@ -122,7 +216,7 @@ public final class NPathComplexityCheck extends AbstractCheck {
     public void beginTree(DetailAST rootAST) {
         rangeValues.clear();
         expressionValues.clear();
-        isAfterValues.clear();
+        afterValues.clear();
         processingTokenEnd.reset();
         currentRangeValue = INITIAL_VALUE;
         branchVisited = false;
@@ -237,7 +331,7 @@ public final class NPathComplexityCheck extends AbstractCheck {
      */
     private void visitUnitaryOperator(DetailAST ast, int basicBranchingFactor) {
         final boolean isAfter = processingTokenEnd.isAfter(ast);
-        isAfterValues.push(isAfter);
+        afterValues.push(isAfter);
         if (!isAfter) {
             processingTokenEnd.setToken(getLastToken(ast));
             final int expressionValue = basicBranchingFactor + countConditionalOperators(ast);
@@ -249,7 +343,7 @@ public final class NPathComplexityCheck extends AbstractCheck {
      * Leaves ternary operator (?:) and return tokens.
      */
     private void leaveUnitaryOperator() {
-        if (!isAfterValues.pop()) {
+        if (!afterValues.pop()) {
             final Values valuePair = popValue();
             BigInteger basicRangeValue = valuePair.getRangeValue();
             BigInteger expressionValue = valuePair.getExpressionValue();
@@ -458,7 +552,7 @@ public final class NPathComplexityCheck extends AbstractCheck {
          * @param valueOfRange NP value for range
          * @param valueOfExpression NP value for expression
          */
-        Values(BigInteger valueOfRange, BigInteger valueOfExpression) {
+        /* package */ Values(BigInteger valueOfRange, BigInteger valueOfExpression) {
             rangeValue = valueOfRange;
             expressionValue = valueOfExpression;
         }
