@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -29,21 +29,42 @@ import com.puppycrawl.tools.checkstyle.api.TokenTypes;
 import com.puppycrawl.tools.checkstyle.utils.CommonUtil;
 
 /**
- * Checks for redundant modifiers in interface and annotation definitions,
- * final modifier on methods of final classes, inner {@code interface}
- * declarations that are declared as {@code static}, non public class
- * constructors and enum constructors, nested enum definitions that are declared
- * as {@code static}.
- *
- * <p>Interfaces by definition are abstract so the {@code abstract}
+ * <p>
+ * Checks for redundant modifiers.
+ * </p>
+ * <p>
+ * Rationale: The Java Language Specification strongly discourages the usage
+ * of {@code public} and {@code abstract} for method declarations in interface
+ * definitions as a matter of style.
+ * </p>
+ * <p>The check validates:</p>
+ * <ol>
+ * <li>
+ * Interface and annotation definitions.
+ * </li>
+ * <li>
+ * Final modifier on methods of final and anonymous classes.
+ * </li>
+ * <li>
+ * Inner {@code interface} declarations that are declared as {@code static}.
+ * </li>
+ * <li>
+ * Class constructors.
+ * </li>
+ * <li>
+ * Nested {@code enum} definitions that are declared as {@code static}.
+ * </li>
+ * </ol>
+ * <p>
+ * Interfaces by definition are abstract so the {@code abstract}
  * modifier on the interface is redundant.
- *
+ * </p>
  * <p>Classes inside of interfaces by definition are public and static,
  * so the {@code public} and {@code static} modifiers
  * on the inner classes are redundant. On the other hand, classes
  * inside of interfaces can be abstract or non abstract.
  * So, {@code abstract} modifier is allowed.
- *
+ * </p>
  * <p>Fields in interfaces and annotations are automatically
  * public, static and final, so these modifiers are redundant as
  * well.</p>
@@ -61,33 +82,35 @@ import com.puppycrawl.tools.checkstyle.utils.CommonUtil;
  * See the following example:</p>
  * <pre>
  * public enum EnumClass {
- *    FIELD_1,
- *    FIELD_2 {
- *        &#64;Override
- *        public final void method1() {} // violation expected
- *    };
+ *   FIELD_1,
+ *   FIELD_2 {
+ *     &#64;Override
+ *     public final void method1() {} // violation expected
+ *   };
  *
- *    public void method1() {}
- *    public final void method2() {} // no violation expected
+ *   public void method1() {}
+ *   public final void method2() {} // no violation expected
  * }
  * </pre>
  *
  * <p>Since these methods can be overridden in these situations, the final methods are not
  * marked as redundant even though they can't be extended by other classes/enums.</p>
- *
+ * <p>
+ * Nested {@code enum} types are always static by default.
+ * </p>
  * <p>Final classes by definition cannot be extended so the {@code final}
  * modifier on the method of a final class is redundant.
- *
+ * </p>
  * <p>Public modifier for constructors in non-public non-protected classes
  * is always obsolete: </p>
  *
  * <pre>
  * public class PublicClass {
- *     public PublicClass() {} // OK
+ *   public PublicClass() {} // OK
  * }
  *
  * class PackagePrivateClass {
- *     public PackagePrivateClass() {} // violation expected
+ *   public PackagePrivateClass() {} // violation expected
  * }
  * </pre>
  *
@@ -98,18 +121,55 @@ import com.puppycrawl.tools.checkstyle.utils.CommonUtil;
  * <pre>
  * package a;
  * public class ClassExample {
- *     protected class ProtectedInnerClass {
- *         public ProtectedInnerClass () {}
- *     }
+ *   protected class ProtectedInnerClass {
+ *     public ProtectedInnerClass () {}
+ *   }
  * }
  *
  * package b;
  * import a.ClassExample;
  * public class ClassExtending extends ClassExample {
- *     ProtectedInnerClass pc = new ProtectedInnerClass();
+ *   ProtectedInnerClass pc = new ProtectedInnerClass();
  * }
  * </pre>
+ * <ul>
+ * <li>
+ * Property {@code tokens} - tokens to check
+ * Default value is:
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#METHOD_DEF">
+ * METHOD_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#VARIABLE_DEF">
+ * VARIABLE_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#ANNOTATION_FIELD_DEF">
+ * ANNOTATION_FIELD_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#INTERFACE_DEF">
+ * INTERFACE_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#CTOR_DEF">
+ * CTOR_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#CLASS_DEF">
+ * CLASS_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#ENUM_DEF">
+ * ENUM_DEF</a>,
+ * <a href="https://checkstyle.org/apidocs/com/puppycrawl/tools/checkstyle/api/TokenTypes.html#RESOURCE">
+ * RESOURCE</a>.
+ * </li>
+ * </ul>
+ * <p>
+ * To configure the check:
+ * </p>
+ * <pre>
+ * &lt;module name="RedundantModifier"/&gt;
+ * </pre>
+ * <p>
+ * To configure the check to check only methods and not variables:
+ * </p>
+ * <pre>
+ * &lt;module name="RedundantModifier"&gt;
+ *   &lt;property name="tokens" value="METHOD_DEF"/&gt;
+ * &lt;/module&gt;
+ * </pre>
  *
+ * @since 3.0
  */
 @StatelessCheck
 public class RedundantModifierCheck
@@ -236,7 +296,7 @@ public class RedundantModifierCheck
         if (isInterfaceOrAnnotationMember(ast)) {
             processInterfaceOrAnnotation(ast);
         }
-        else if (ast.getParent() != null) {
+        else {
             checkForRedundantModifier(ast, TokenTypes.LITERAL_STATIC);
         }
     }
