@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -37,36 +37,211 @@ import com.puppycrawl.tools.checkstyle.utils.CommonUtil;
 
 /**
  * <p>
- * A filter that uses nearby comments to suppress audit events.
+ * Filter {@code SuppressWithNearbyCommentFilter} uses nearby comments to suppress audit events.
  * </p>
+ * <p>
+ * Rationale: Same as {@code SuppressionCommentFilter}.
+ * Whereas the SuppressionCommentFilter uses matched pairs of filters to turn
+ * on/off comment matching, {@code SuppressWithNearbyCommentFilter} uses single comments.
+ * This requires fewer lines to mark a region, and may be aesthetically preferable in some contexts.
+ * </p>
+ * <p>
+ * Attention: This filter may only be specified within the TreeWalker module
+ * ({@code &lt;module name="TreeWalker"/&gt;}) and only applies to checks which are also
+ * defined within this module. To filter non-TreeWalker checks like {@code RegexpSingleline},
+ * a <a href="https://checkstyle.org/config_filters.html#SuppressWithPlainTextCommentFilter">
+ * SuppressWithPlainTextCommentFilter</a> or similar filter must be used.
+ * </p>
+ * <ul>
+ * <li>
+ * Property {@code commentFormat} - Specify comment pattern to trigger filter to begin suppression.
+ * Default value is {@code "SUPPRESS CHECKSTYLE (\w+)"}.
+ * </li>
+ * <li>
+ * Property {@code checkFormat} - Specify check pattern to suppress.
+ * Default value is {@code ".*"}.
+ * </li>
+ * <li>
+ * Property {@code messageFormat} - Define message pattern to suppress.
+ * Default value is {@code null}.
+ * </li>
+ * <li>
+ * Property {@code idFormat} - Specify check ID pattern to suppress.
+ * Default value is {@code null}.
+ * </li>
+ * <li>
+ * Property {@code influenceFormat} - Specify negative/zero/positive value that
+ * defines the number of lines preceding/at/following the suppression comment.
+ * Default value is {@code "0"}.
+ * </li>
+ * <li>
+ * Property {@code checkCPP} - Control whether to check C++ style comments ({@code //}).
+ * Default value is {@code true}.
+ * </li>
+ * <li>
+ * Property {@code checkC} - Control whether to check C style comments ({@code &#47;* ... *&#47;}).
+ * Default value is {@code true}.
+ * </li>
+ * </ul>
+ * <p>
+ * To configure a filter to suppress audit events for <i>check</i> on any line
+ * with a comment {@code SUPPRESS CHECKSTYLE <i>check</i>}:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;SuppressWithNearbyCommentFilter&quot;/&gt;
+ * </pre>
+ * <pre>
+ * private int [] array; // SUPPRESS CHECKSTYLE
+ * </pre>
+ * <p>
+ * To configure a filter to suppress all audit events on any line containing
+ * the comment {@code CHECKSTYLE IGNORE THIS LINE}:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;SuppressWithNearbyCommentFilter&quot;&gt;
+ *   &lt;property name=&quot;commentFormat&quot; value=&quot;CHECKSTYLE IGNORE THIS LINE&quot;/&gt;
+ *   &lt;property name=&quot;checkFormat&quot; value=&quot;.*&quot;/&gt;
+ *   &lt;property name=&quot;influenceFormat&quot; value=&quot;0&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * public static final int lowerCaseConstant; // CHECKSTYLE IGNORE THIS LINE
+ * </pre>
+ * <p>
+ * To configure a filter so that {@code // OK to catch (Throwable|Exception|RuntimeException) here}
+ * permits the current and previous line to avoid generating an IllegalCatch audit event:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;SuppressWithNearbyCommentFilter&quot;&gt;
+ *   &lt;property name=&quot;commentFormat&quot; value=&quot;OK to catch (\w+) here&quot;/&gt;
+ *   &lt;property name=&quot;checkFormat&quot; value=&quot;IllegalCatchCheck&quot;/&gt;
+ *   &lt;property name=&quot;messageFormat&quot; value=&quot;$1&quot;/&gt;
+ *   &lt;property name=&quot;influenceFormat&quot; value=&quot;-1&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * . . .
+ * catch (RuntimeException re) {
+ * // OK to catch RuntimeException here
+ * }
+ * catch (Throwable th) { ... }
+ * . . .
+ * </pre>
+ * <p>
+ * To configure a filter so that {@code CHECKSTYLE IGNORE <i>check</i> FOR NEXT
+ * <i>var</i> LINES} avoids triggering any audits for the given check for
+ * the current line and the next <i>var</i> lines (for a total of <i>var</i>+1 lines):
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;SuppressWithNearbyCommentFilter&quot;&gt;
+ *   &lt;property name=&quot;commentFormat&quot;
+ *       value=&quot;CHECKSTYLE IGNORE (\w+) FOR NEXT (\d+) LINES&quot;/&gt;
+ *   &lt;property name=&quot;checkFormat&quot; value=&quot;$1&quot;/&gt;
+ *   &lt;property name=&quot;influenceFormat&quot; value=&quot;$2&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * static final int lowerCaseConstant; // CHECKSTYLE IGNORE ConstantNameCheck FOR NEXT 3 LINES
+ * static final int lowerCaseConstant1;
+ * static final int lowerCaseConstant2;
+ * static final int lowerCaseConstant3;
+ * static final int lowerCaseConstant4; // will warn here
+ * </pre>
+ * <p>
+ * To configure a filter to avoid any audits on code like:
+ * </p>
+ * <pre>
+ * &lt;module name=&quot;SuppressWithNearbyCommentFilter&quot;&gt;
+ *   &lt;property name=&quot;commentFormat&quot;
+ *     value=&quot;ALLOW (\\w+) ON PREVIOUS LINE&quot;/&gt;
+ *   &lt;property name=&quot;checkFormat&quot; value=&quot;$1&quot;/&gt;
+ *   &lt;property name=&quot;influenceFormat&quot; value=&quot;-1&quot;/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * private int D2;
+ * // ALLOW MemberName ON PREVIOUS LINE
+ * . . .
+ * </pre>
+ * <p>
+ * To configure a filter to allow suppress one or more Checks (separated by "|")
+ * and demand comment no less than 14 symbols:
+ * </p>
+ * <pre>
+ * &lt;module name="SuppressWithNearbyCommentFilter"&gt;
+ *   &lt;property name="commentFormat"
+ *     value="@cs\.suppress \[(\w+(\|\w+)*)\] \w[-\.'`,:;\w ]{14,}"/&gt;
+ *   &lt;property name="checkFormat" value="$1"/&gt;
+ *   &lt;property name="influenceFormat" value="1"/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * public static final int [] array; // @cs.suppress [ConstantName|NoWhitespaceAfter] A comment here
+ * </pre>
+ * <p>
+ * It is possible to specify an ID of checks, so that it can be leveraged by
+ * the SuppressWithNearbyCommentFilter to skip validations. The following examples show how to skip
+ * validations near code that has comment like {@code // @cs-: &lt;ID/&gt; (reason)},
+ * where ID is the ID of checks you want to suppress.
+ * </p>
+ * <p>
+ * Examples of Checkstyle checks configuration:
+ * </p>
+ * <pre>
+ * &lt;module name="RegexpSinglelineJava"&gt;
+ *   &lt;property name="id" value="ignore"/&gt;
+ *   &lt;property name="format" value="^.*@Ignore\s*$"/&gt;
+ *   &lt;property name="message" value="@Ignore should have a reason."/&gt;
+ * &lt;/module&gt;
  *
- * <p>This check is philosophically similar to {@link SuppressionCommentFilter}.
- * Unlike {@link SuppressionCommentFilter}, this filter does not require
- * pairs of comments.  This check may be used to suppress warnings in the
- * current line:
- * <pre>
- *    offendingLine(for, whatever, reason); // SUPPRESS ParameterNumberCheck
+ * &lt;module name="RegexpSinglelineJava"&gt;
+ *   &lt;property name="id" value="systemout"/&gt;
+ *   &lt;property name="format" value="^.*System\.(out|err).*$"/&gt;
+ *   &lt;property name="message" value="Don't use System.out/err, use SLF4J instead."/&gt;
+ * &lt;/module&gt;
  * </pre>
- * or it may be configured to span multiple lines, either forward:
+ * <p>
+ * Example of SuppressWithNearbyCommentFilter configuration (idFormat which is set to
+ * '$1' points that ID of the checks is in the first group of commentFormat regular expressions):
+ * </p>
  * <pre>
- *    // PERMIT MultipleVariableDeclarations NEXT 3 LINES
- *    double x1 = 1.0, y1 = 0.0, z1 = 0.0;
- *    double x2 = 0.0, y2 = 1.0, z2 = 0.0;
- *    double x3 = 0.0, y3 = 0.0, z3 = 1.0;
+ * &lt;module name="SuppressWithNearbyCommentFilter"&gt;
+ *   &lt;property name="commentFormat" value="@cs-: (\w+) \(.*\)"/&gt;
+ *   &lt;property name="idFormat" value="$1"/&gt;
+ *   &lt;property name="influenceFormat" value="0"/&gt;
+ * &lt;/module&gt;
  * </pre>
- * or reverse:
  * <pre>
- *   try {
- *     thirdPartyLibrary.method();
- *   } catch (RuntimeException ex) {
- *     // ALLOW ILLEGAL CATCH BECAUSE third party API wraps everything
- *     // in RuntimeExceptions.
- *     ...
- *   }
+ * &#64;Ignore // @cs-: ignore (test has not been implemented yet)
+ * &#64;Test
+ * public void testMethod() { }
+ *
+ * public static void foo() {
+ *   System.out.println("Debug info."); // @cs-: systemout (should not fail RegexpSinglelineJava)
+ * }
+ * </pre>
+ * <p>
+ * Example of how to configure the check to suppress more than one checks.
+ * The influence format format is specified in the second regexp group.
+ * </p>
+ * <pre>
+ * &lt;module name="SuppressWithNearbyCommentFilter"&gt;
+ *   &lt;property name="commentFormat" value="@cs-\: ([\w\|]+) influence (\d+)"/&gt;
+ *   &lt;property name="checkFormat" value="$1"/&gt;
+ *   &lt;property name="influenceFormat" value="$2"/&gt;
+ * &lt;/module&gt;
+ * </pre>
+ * <pre>
+ * // @cs-: ClassDataAbstractionCoupling influence 2
+ * // @cs-: MagicNumber influence 4
+ * &#64;Service // no violations from ClassDataAbstractionCoupling here
+ * &#64;Transactional
+ * public class UserService {
+ *   private int value = 10022; // no violations from MagicNumber here
+ * }
  * </pre>
  *
- * <p>See {@link SuppressionCommentFilter} for usage notes.
- *
+ * @since 5.0
  */
 public class SuppressWithNearbyCommentFilter
     extends AutomaticBean
@@ -85,24 +260,30 @@ public class SuppressWithNearbyCommentFilter
     /** Tagged comments. */
     private final List<Tag> tags = new ArrayList<>();
 
-    /** Whether to look for trigger in C-style comments. */
+    /** Control whether to check C style comments ({@code &#47;* ... *&#47;}). */
     private boolean checkC = true;
 
-    /** Whether to look for trigger in C++-style comments. */
+    /** Control whether to check C++ style comments ({@code //}). */
     // -@cs[AbbreviationAsWordInName] We can not change it as,
     // check's property is a part of API (used in configurations).
     private boolean checkCPP = true;
 
-    /** Parsed comment regexp that marks checkstyle suppression region. */
+    /** Specify comment pattern to trigger filter to begin suppression. */
     private Pattern commentFormat = Pattern.compile(DEFAULT_COMMENT_FORMAT);
 
-    /** The comment pattern that triggers suppression. */
+    /** Specify check pattern to suppress. */
     private String checkFormat = DEFAULT_CHECK_FORMAT;
 
-    /** The message format to suppress. */
+    /** Define message pattern to suppress. */
     private String messageFormat;
 
-    /** The influence of the suppression comment. */
+    /** Specify check ID pattern to suppress. */
+    private String idFormat;
+
+    /**
+     * Specify negative/zero/positive value that defines the number of lines
+     * preceding/at/following the suppression comment.
+     */
     private String influenceFormat = DEFAULT_INFLUENCE_FORMAT;
 
     /**
@@ -115,7 +296,7 @@ public class SuppressWithNearbyCommentFilter
     private WeakReference<FileContents> fileContentsReference = new WeakReference<>(null);
 
     /**
-     * Set the format for a comment that turns off reporting.
+     * Setter to specify comment pattern to trigger filter to begin suppression.
      * @param pattern a pattern.
      */
     public final void setCommentFormat(Pattern pattern) {
@@ -140,7 +321,7 @@ public class SuppressWithNearbyCommentFilter
     }
 
     /**
-     * Set the format for a check.
+     * Setter to specify check pattern to suppress.
      * @param format a {@code String} value
      */
     public final void setCheckFormat(String format) {
@@ -148,7 +329,7 @@ public class SuppressWithNearbyCommentFilter
     }
 
     /**
-     * Set the format for a message.
+     * Setter to define message pattern to suppress.
      * @param format a {@code String} value
      */
     public void setMessageFormat(String format) {
@@ -156,7 +337,16 @@ public class SuppressWithNearbyCommentFilter
     }
 
     /**
-     * Set the format for the influence of this check.
+     * Setter to specify check ID pattern to suppress.
+     * @param format a {@code String} value
+     */
+    public void setIdFormat(String format) {
+        idFormat = format;
+    }
+
+    /**
+     * Setter to specify negative/zero/positive value that defines the number
+     * of lines preceding/at/following the suppression comment.
      * @param format a {@code String} value
      */
     public final void setInfluenceFormat(String format) {
@@ -164,7 +354,7 @@ public class SuppressWithNearbyCommentFilter
     }
 
     /**
-     * Set whether to look in C++ comments.
+     * Setter to control whether to check C++ style comments ({@code //}).
      * @param checkCpp {@code true} if C++ comments are checked.
      */
     // -@cs[AbbreviationAsWordInName] We can not change it as,
@@ -174,7 +364,7 @@ public class SuppressWithNearbyCommentFilter
     }
 
     /**
-     * Set whether to look in C comments.
+     * Setter to control whether to check C style comments ({@code &#47;* ... *&#47;}).
      * @param checkC {@code true} if C comments are checked.
      */
     public void setCheckC(boolean checkC) {
@@ -281,7 +471,7 @@ public class SuppressWithNearbyCommentFilter
     /**
      * A Tag holds a suppression comment and its location.
      */
-    public static class Tag {
+    private static final class Tag {
 
         /** The text of the tag. */
         private final String text;
@@ -298,6 +488,9 @@ public class SuppressWithNearbyCommentFilter
         /** The parsed message regexp, expanded for the text of this tag. */
         private final Pattern tagMessageRegexp;
 
+        /** The parsed check ID regexp, expanded for the text of this tag. */
+        private final Pattern tagIdRegexp;
+
         /**
          * Constructs a tag.
          * @param text the text of the suppression.
@@ -305,7 +498,7 @@ public class SuppressWithNearbyCommentFilter
          * @param filter the {@code SuppressWithNearbyCommentFilter} with the context
          * @throws IllegalArgumentException if unable to parse expanded text.
          */
-        public Tag(String text, int line, SuppressWithNearbyCommentFilter filter) {
+        /* package */ Tag(String text, int line, SuppressWithNearbyCommentFilter filter) {
             this.text = text;
 
             //Expand regexp for check and message
@@ -323,12 +516,17 @@ public class SuppressWithNearbyCommentFilter
                             filter.messageFormat, text, filter.commentFormat);
                     tagMessageRegexp = Pattern.compile(format);
                 }
+                if (filter.idFormat == null) {
+                    tagIdRegexp = null;
+                }
+                else {
+                    format = CommonUtil.fillTemplateWithStringsByRegexp(
+                            filter.idFormat, text, filter.commentFormat);
+                    tagIdRegexp = Pattern.compile(format);
+                }
                 format = CommonUtil.fillTemplateWithStringsByRegexp(
                         filter.influenceFormat, text, filter.commentFormat);
 
-                if (CommonUtil.startsWithChar(format, '+')) {
-                    format = format.substring(1);
-                }
                 final int influence = parseInfluence(format, filter.influenceFormat, text);
 
                 if (influence >= 1) {
@@ -377,12 +575,14 @@ public class SuppressWithNearbyCommentFilter
                     && Objects.equals(lastLine, tag.lastLine)
                     && Objects.equals(text, tag.text)
                     && Objects.equals(tagCheckRegexp, tag.tagCheckRegexp)
-                    && Objects.equals(tagMessageRegexp, tag.tagMessageRegexp);
+                    && Objects.equals(tagMessageRegexp, tag.tagMessageRegexp)
+                    && Objects.equals(tagIdRegexp, tag.tagIdRegexp);
         }
 
         @Override
         public int hashCode() {
-            return Objects.hash(text, firstLine, lastLine, tagCheckRegexp, tagMessageRegexp);
+            return Objects.hash(text, firstLine, lastLine, tagCheckRegexp, tagMessageRegexp,
+                    tagIdRegexp);
         }
 
         /**
@@ -392,29 +592,65 @@ public class SuppressWithNearbyCommentFilter
          * @return true if the source of event matches the text of this tag.
          */
         public boolean isMatch(TreeWalkerAuditEvent event) {
+            return isInScopeOfSuppression(event)
+                    && isCheckMatch(event)
+                    && isIdMatch(event)
+                    && isMessageMatch(event);
+        }
+
+        /**
+         * Checks whether the {@link TreeWalkerAuditEvent} is in the scope of the suppression.
+         * @param event {@link TreeWalkerAuditEvent} instance.
+         * @return true if the {@link TreeWalkerAuditEvent} is in the scope of the suppression.
+         */
+        private boolean isInScopeOfSuppression(TreeWalkerAuditEvent event) {
             final int line = event.getLine();
-            boolean match = false;
+            return line >= firstLine && line <= lastLine;
+        }
 
-            if (line >= firstLine && line <= lastLine) {
-                final Matcher tagMatcher = tagCheckRegexp.matcher(event.getSourceName());
+        /**
+         * Checks whether {@link TreeWalkerAuditEvent} source name matches the check format.
+         * @param event {@link TreeWalkerAuditEvent} instance.
+         * @return true if the {@link TreeWalkerAuditEvent} source name matches the check format.
+         */
+        private boolean isCheckMatch(TreeWalkerAuditEvent event) {
+            final Matcher checkMatcher = tagCheckRegexp.matcher(event.getSourceName());
+            return checkMatcher.find();
+        }
 
-                if (tagMatcher.find()) {
-                    match = true;
-                }
-                else if (tagMessageRegexp == null) {
-                    if (event.getModuleId() != null) {
-                        final Matcher idMatcher = tagCheckRegexp.matcher(event.getModuleId());
-                        match = idMatcher.find();
-                    }
+        /**
+         * Checks whether the {@link TreeWalkerAuditEvent} module ID matches the ID format.
+         * @param event {@link TreeWalkerAuditEvent} instance.
+         * @return true if the {@link TreeWalkerAuditEvent} module ID matches the ID format.
+         */
+        private boolean isIdMatch(TreeWalkerAuditEvent event) {
+            boolean match = true;
+            if (tagIdRegexp != null) {
+                if (event.getModuleId() == null) {
+                    match = false;
                 }
                 else {
-                    final Matcher messageMatcher = tagMessageRegexp.matcher(event.getMessage());
-                    match = messageMatcher.find();
+                    final Matcher idMatcher = tagIdRegexp.matcher(event.getModuleId());
+                    match = idMatcher.find();
                 }
             }
             return match;
         }
 
+        /**
+         * Checks whether the {@link TreeWalkerAuditEvent} message matches the message format.
+         * @param event {@link TreeWalkerAuditEvent} instance.
+         * @return true if the {@link TreeWalkerAuditEvent} message matches the message format.
+         */
+        private boolean isMessageMatch(TreeWalkerAuditEvent event) {
+            boolean match = true;
+            if (tagMessageRegexp != null) {
+                final Matcher messageMatcher = tagMessageRegexp.matcher(event.getMessage());
+                match = messageMatcher.find();
+            }
+            return match;
+        }
+
         @Override
         public String toString() {
             return "Tag[text='" + text + '\''
@@ -422,6 +658,7 @@ public class SuppressWithNearbyCommentFilter
                     + ", lastLine=" + lastLine
                     + ", tagCheckRegexp=" + tagCheckRegexp
                     + ", tagMessageRegexp=" + tagMessageRegexp
+                    + ", tagIdRegexp=" + tagIdRegexp
                     + ']';
         }
 
