@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -36,9 +36,8 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
 
-import org.apache.commons.beanutils.ConversionException;
-
 import antlr.Token;
+import com.puppycrawl.tools.checkstyle.DetailAstImpl;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
 import com.puppycrawl.tools.checkstyle.api.DetailAST;
 import com.puppycrawl.tools.checkstyle.api.TokenTypes;
@@ -49,6 +48,9 @@ import com.puppycrawl.tools.checkstyle.api.TokenTypes;
  */
 public final class CommonUtil {
 
+    /** Default tab width for column reporting. */
+    public static final int DEFAULT_TAB_WIDTH = 8;
+
     /** Copied from org.apache.commons.lang3.ArrayUtils. */
     public static final String[] EMPTY_STRING_ARRAY = new String[0];
     /** Copied from org.apache.commons.lang3.ArrayUtils. */
@@ -82,7 +84,7 @@ public final class CommonUtil {
      * @param pattern
      *            the pattern to match
      * @return a created regexp object
-     * @throws ConversionException
+     * @throws IllegalArgumentException
      *             if unable to create Pattern object.
      **/
     public static Pattern createPattern(String pattern) {
@@ -116,13 +118,13 @@ public final class CommonUtil {
      * @return DetailAST block comment
      */
     public static DetailAST createBlockCommentNode(String content) {
-        final DetailAST blockCommentBegin = new DetailAST();
+        final DetailAstImpl blockCommentBegin = new DetailAstImpl();
         blockCommentBegin.setType(TokenTypes.BLOCK_COMMENT_BEGIN);
         blockCommentBegin.setText(BLOCK_MULTIPLE_COMMENT_BEGIN);
         blockCommentBegin.setLineNo(0);
         blockCommentBegin.setColumnNo(-JAVADOC_START.length());
 
-        final DetailAST commentContent = new DetailAST();
+        final DetailAstImpl commentContent = new DetailAstImpl();
         commentContent.setType(TokenTypes.COMMENT_CONTENT);
         commentContent.setText("*" + content);
         commentContent.setLineNo(0);
@@ -130,7 +132,7 @@ public final class CommonUtil {
         // that contains javadoc identifier has -1 column
         commentContent.setColumnNo(-1);
 
-        final DetailAST blockCommentEnd = new DetailAST();
+        final DetailAstImpl blockCommentEnd = new DetailAstImpl();
         blockCommentEnd.setType(TokenTypes.BLOCK_COMMENT_END);
         blockCommentEnd.setText(BLOCK_MULTIPLE_COMMENT_END);
 
@@ -146,14 +148,14 @@ public final class CommonUtil {
      * @return DetailAST with BLOCK_COMMENT type.
      */
     public static DetailAST createBlockCommentNode(Token token) {
-        final DetailAST blockComment = new DetailAST();
+        final DetailAstImpl blockComment = new DetailAstImpl();
         blockComment.initialize(TokenTypes.BLOCK_COMMENT_BEGIN, BLOCK_MULTIPLE_COMMENT_BEGIN);
 
         // column counting begins from 0
         blockComment.setColumnNo(token.getColumn() - 1);
         blockComment.setLineNo(token.getLine());
 
-        final DetailAST blockCommentContent = new DetailAST();
+        final DetailAstImpl blockCommentContent = new DetailAstImpl();
         blockCommentContent.setType(TokenTypes.COMMENT_CONTENT);
 
         // column counting begins from 0
@@ -162,7 +164,7 @@ public final class CommonUtil {
         blockCommentContent.setLineNo(token.getLine());
         blockCommentContent.setText(token.getText());
 
-        final DetailAST blockCommentClose = new DetailAST();
+        final DetailAstImpl blockCommentClose = new DetailAstImpl();
         blockCommentClose.initialize(TokenTypes.BLOCK_COMMENT_END, BLOCK_MULTIPLE_COMMENT_END);
 
         final Map.Entry<Integer, Integer> linesColumns = countLinesColumns(
@@ -504,8 +506,13 @@ public final class CommonUtil {
             else {
                 // check to see if the file is in the classpath
                 try {
-                    final URL configUrl = CommonUtil.class
-                            .getResource(filename);
+                    final URL configUrl;
+                    if (filename.charAt(0) == '/') {
+                        configUrl = CommonUtil.class.getResource(filename);
+                    }
+                    else {
+                        configUrl = ClassLoader.getSystemResource(filename);
+                    }
                     if (configUrl == null) {
                         throw new CheckstyleException(UNABLE_TO_FIND_EXCEPTION_PREFIX + filename);
                     }
