@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,9 @@
 package com.puppycrawl.tools.checkstyle;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-import static org.powermock.api.mockito.PowerMockito.when;
 
 import java.io.File;
 import java.lang.reflect.Constructor;
@@ -35,22 +35,17 @@ import java.util.List;
 import java.util.Properties;
 
 import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.powermock.api.mockito.PowerMockito;
-import org.powermock.core.classloader.annotations.PrepareForTest;
-import org.powermock.modules.junit4.PowerMockRunner;
 import org.powermock.reflect.Whitebox;
-import org.xml.sax.Attributes;
 import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
 
+import com.puppycrawl.tools.checkstyle.ConfigurationLoader.IgnoredModulesOptions;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
 import com.puppycrawl.tools.checkstyle.api.Configuration;
 
 /**
  * Unit test for ConfigurationLoader.
  */
-@RunWith(PowerMockRunner.class)
-@PrepareForTest({DefaultConfiguration.class, ConfigurationLoader.class})
 public class ConfigurationLoaderTest extends AbstractPathTestSupport {
 
     @Override
@@ -183,7 +178,7 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         try {
             final String fName = getPath("InputConfigurationLoaderMissingPropertyName.xml");
             ConfigurationLoader.loadConfiguration(fName, new PropertiesExpander(new Properties()),
-                    false);
+                    IgnoredModulesOptions.EXECUTE);
 
             fail("missing property name");
         }
@@ -413,6 +408,9 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         final Properties props = new Properties();
         props.setProperty("checkstyle.basedir", "basedir");
 
+        System.setProperty(
+                XmlLoader.LoadExternalDtdFeatureProvider.ENABLE_EXTERNAL_DTD_LOAD, "true");
+
         final DefaultConfiguration config =
             (DefaultConfiguration) loadConfiguration(
                 "InputConfigurationLoaderExternalEntity.xml", props);
@@ -428,6 +426,9 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         final Properties props = new Properties();
         props.setProperty("checkstyle.basedir", "basedir");
 
+        System.setProperty(
+                XmlLoader.LoadExternalDtdFeatureProvider.ENABLE_EXTERNAL_DTD_LOAD, "true");
+
         final DefaultConfiguration config =
             (DefaultConfiguration) loadConfiguration(
                 "subdir/InputConfigurationLoaderExternalEntitySubDir.xml", props);
@@ -443,6 +444,9 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         final Properties props = new Properties();
         props.setProperty("checkstyle.basedir", "basedir");
 
+        System.setProperty(
+                XmlLoader.LoadExternalDtdFeatureProvider.ENABLE_EXTERNAL_DTD_LOAD, "true");
+
         final File file = new File(
                 getPath("subdir/InputConfigurationLoaderExternalEntitySubDir.xml"));
         final DefaultConfiguration config =
@@ -457,33 +461,27 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
 
     @Test
     public void testIncorrectTag() throws Exception {
+        final Class<?> aClassParent = ConfigurationLoader.class;
+        final Constructor<?> ctorParent = aClassParent.getDeclaredConstructor(
+                PropertyResolver.class, boolean.class, ThreadModeSettings.class);
+        ctorParent.setAccessible(true);
+        final Object objParent = ctorParent.newInstance(null, true, null);
+
+        final Class<?> aClass = Class.forName("com.puppycrawl.tools.checkstyle."
+                + "ConfigurationLoader$InternalLoader");
+        final Constructor<?> constructor = aClass.getDeclaredConstructor(objParent.getClass());
+        constructor.setAccessible(true);
+
+        final Object obj = constructor.newInstance(objParent);
+
         try {
-            final Class<?> aClassParent = ConfigurationLoader.class;
-            final Constructor<?> ctorParent = aClassParent.getDeclaredConstructor(
-                    PropertyResolver.class, boolean.class, ThreadModeSettings.class);
-            ctorParent.setAccessible(true);
-            final Object objParent = ctorParent.newInstance(null, true, null);
-
-            final Class<?> aClass = Class.forName("com.puppycrawl.tools.checkstyle."
-                    + "ConfigurationLoader$InternalLoader");
-            final Constructor<?> constructor = aClass.getConstructor(objParent.getClass());
-            constructor.setAccessible(true);
-
-            final Object obj = constructor.newInstance(objParent);
-
-            final Class<?>[] param = new Class<?>[] {String.class, String.class,
-                String.class, Attributes.class, };
-            final Method method = aClass.getDeclaredMethod("startElement", param);
-
-            method.invoke(obj, "", "", "hello", null);
+            Whitebox.invokeMethod(obj, "startElement", "", "", "hello", null);
 
             fail("Exception is expected");
         }
-        catch (InvocationTargetException ex) {
-            assertTrue("Invalid exception cause",
-                ex.getCause() instanceof IllegalStateException);
+        catch (IllegalStateException ex) {
             assertEquals("Invalid exception cause message",
-                "Unknown name:" + "hello" + ".", ex.getCause().getMessage());
+                "Unknown name:" + "hello" + ".", ex.getMessage());
         }
     }
 
@@ -496,9 +494,13 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         catch (CheckstyleException ex) {
             assertEquals("Invalid exception message",
                 "unable to parse configuration stream", ex.getMessage());
+            assertSame("Expected cause of type SAXException",
+                SAXException.class, ex.getCause().getClass());
+            assertSame("Expected cause of type CheckstyleException",
+                CheckstyleException.class, ex.getCause().getCause().getClass());
             assertEquals("Invalid exception cause message",
                 "Property ${nonexistent} has not been set",
-                    ex.getCause().getMessage());
+                ex.getCause().getCause().getMessage());
         }
     }
 
@@ -507,7 +509,7 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         final DefaultConfiguration config =
                 (DefaultConfiguration) ConfigurationLoader.loadConfiguration(
                         getPath("InputConfigurationLoaderModuleIgnoreSeverity.xml"),
-                        new PropertiesExpander(new Properties()), true);
+                        new PropertiesExpander(new Properties()), IgnoredModulesOptions.OMIT);
 
         final Configuration[] children = config.getChildren();
         assertEquals("Invalid children count", 0, children[0].getChildren().length);
@@ -519,7 +521,7 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
                 (DefaultConfiguration) ConfigurationLoader.loadConfiguration(new InputSource(
                         new File(getPath("InputConfigurationLoaderModuleIgnoreSeverity.xml"))
                             .toURI().toString()),
-                        new PropertiesExpander(new Properties()), true);
+                        new PropertiesExpander(new Properties()), IgnoredModulesOptions.OMIT);
 
         final Configuration[] children = config.getChildren();
         assertEquals("Invalid children count", 0, children[0].getChildren().length);
@@ -530,7 +532,7 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         final DefaultConfiguration config =
                 (DefaultConfiguration) ConfigurationLoader.loadConfiguration(
                         getPath("InputConfigurationLoaderCheckerIgnoreSeverity.xml"),
-                        new PropertiesExpander(new Properties()), true);
+                        new PropertiesExpander(new Properties()), IgnoredModulesOptions.OMIT);
 
         final Configuration[] children = config.getChildren();
         assertEquals("Invalid children count", 0, children.length);
@@ -542,7 +544,7 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
             final DefaultConfiguration config =
                     (DefaultConfiguration) ConfigurationLoader.loadConfiguration(
                             ";InputConfigurationLoaderModuleIgnoreSeverity.xml",
-                            new PropertiesExpander(new Properties()), true);
+                            new PropertiesExpander(new Properties()), IgnoredModulesOptions.OMIT);
 
             final Configuration[] children = config.getChildren();
             assertEquals("Invalid children count", 0, children[0].getChildren().length);
@@ -559,9 +561,9 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
     public void testLoadConfigurationDeprecated() throws Exception {
         final DefaultConfiguration config =
                 (DefaultConfiguration) ConfigurationLoader.loadConfiguration(
-                        Files.newInputStream(Paths.get(
-                            getPath("InputConfigurationLoaderModuleIgnoreSeverity.xml"))),
-                        new PropertiesExpander(new Properties()), true);
+                        new InputSource(Files.newInputStream(Paths.get(
+                            getPath("InputConfigurationLoaderModuleIgnoreSeverity.xml")))),
+                        new PropertiesExpander(new Properties()), IgnoredModulesOptions.OMIT);
 
         final Configuration[] children = config.getChildren();
         assertEquals("Invalid children count",
@@ -584,44 +586,13 @@ public class ConfigurationLoaderTest extends AbstractPathTestSupport {
         final DefaultConfiguration config =
                 (DefaultConfiguration) ConfigurationLoader.loadConfiguration(
                         getPath("InputConfigurationLoaderModuleIgnoreSeverity.xml"),
-                        new PropertiesExpander(new Properties()), true);
+                        new PropertiesExpander(new Properties()), IgnoredModulesOptions.OMIT);
 
         final Configuration[] children = config.getChildren();
         assertEquals("Invalid children count",
             0, children[0].getChildren().length);
     }
 
-    @Test
-    public void testConfigWithIgnoreExceptionalAttributes() throws Exception {
-        // emulate exception from unrelated code, but that is same try-catch
-        final DefaultConfiguration tested = PowerMockito.mock(DefaultConfiguration.class);
-        when(tested.getAttributeNames()).thenReturn(new String[] {"severity"});
-        when(tested.getName()).thenReturn("MemberName");
-        when(tested.getAttribute("severity")).thenThrow(CheckstyleException.class);
-        // to void creation of 2 other mocks for now reason, only one moc is used for all cases
-        PowerMockito.whenNew(DefaultConfiguration.class)
-                .withArguments("MemberName", ThreadModeSettings.SINGLE_THREAD_MODE_INSTANCE)
-                .thenReturn(tested);
-        PowerMockito.whenNew(DefaultConfiguration.class)
-                .withArguments("Checker", ThreadModeSettings.SINGLE_THREAD_MODE_INSTANCE)
-                .thenReturn(tested);
-        PowerMockito.whenNew(DefaultConfiguration.class)
-                .withArguments("TreeWalker", ThreadModeSettings.SINGLE_THREAD_MODE_INSTANCE)
-                .thenReturn(tested);
-
-        try {
-            ConfigurationLoader.loadConfiguration(
-                    getPath("InputConfigurationLoaderModuleIgnoreSeverity.xml"),
-                    new PropertiesExpander(new Properties()), true);
-            fail("Exception is expected");
-        }
-        catch (CheckstyleException expected) {
-            assertEquals("Invalid exception cause message",
-                "Problem during accessing 'severity' attribute for MemberName",
-                    expected.getCause().getMessage());
-        }
-    }
-
     @Test
     public void testParsePropertyString() throws Exception {
         final List<String> propertyRefs = new ArrayList<>();
