@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -35,7 +35,7 @@ import org.junit.Test;
 import org.junit.contrib.java.lang.system.SystemErrRule;
 import org.junit.contrib.java.lang.system.SystemOutRule;
 
-import antlr.ANTLRException;
+import antlr.MismatchedTokenException;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
 import com.puppycrawl.tools.checkstyle.internal.utils.TestUtil;
 
@@ -44,10 +44,7 @@ public class JavadocPropertiesGeneratorTest extends AbstractPathTestSupport {
     private static final String EOL = System.lineSeparator();
     private static final String USAGE = String.format(Locale.ROOT,
           "Usage: java com.puppycrawl.tools.checkstyle.JavadocPropertiesGenerator [-hV]%n"
-          + "                                                                       "
-          + "--destfile=<outputFile>%n"
-          + "                                                                       "
-          + "<inputFile>%n"
+          + "       --destfile=<outputFile> <inputFile>%n"
           + "      <inputFile>   The input file.%n"
           + "      --destfile=<outputFile>%n"
           + "                    The output file.%n"
@@ -252,10 +249,10 @@ public class JavadocPropertiesGeneratorTest extends AbstractPathTestSupport {
             assertTrue("Invalid error message",
                 ex.getMessage().contains("InputJavadocPropertiesGeneratorParseError.java"));
 
-            final Throwable cause = ex.getCause();
-            assertTrue("Invalid error message", cause instanceof ANTLRException);
+            final Throwable cause = ex.getCause().getCause();
+            assertTrue("Invalid error message", cause instanceof MismatchedTokenException);
             assertTrue("Invalid error message",
-                cause.getMessage().contains("Unexpected character 0x23 in identifier"));
+                cause.getMessage().contains("expecting RCURLY, found '!'"));
         }
     }
 
