@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -19,15 +19,22 @@
 
 package com.puppycrawl.tools.checkstyle.checks.javadoc;
 
-import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_CLASS_INFO;
 import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_DUPLICATE_TAG;
 import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_EXPECTED_TAG;
 import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_INVALID_INHERIT_DOC;
-import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_JAVADOC_MISSING;
 import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_RETURN_EXPECTED;
 import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_UNUSED_TAG;
 import static com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck.MSG_UNUSED_TAG_GENERAL;
 import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 
 import org.junit.Test;
 
@@ -68,34 +75,17 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         final DefaultConfiguration config = createModuleConfig(JavadocMethodCheck.class);
         config.addAttribute("logLoadErrors", "true");
         config.addAttribute("allowUndeclaredRTE", "true");
-        final String[] expected = {
-            "7:8: " + getCheckMessage(MSG_CLASS_INFO, "@throws", "InvalidExceptionName"),
-        };
-        verify(config, getPath("InputJavadocMethodLoadErrors.java"), expected);
+        verify(config, getPath("InputJavadocMethodLoadErrors.java"), CommonUtil.EMPTY_STRING_ARRAY);
     }
 
     @Test
     public void extendAnnotationTest() throws Exception {
         final DefaultConfiguration config = createModuleConfig(JavadocMethodCheck.class);
         config.addAttribute("allowedAnnotations", "MyAnnotation, Override");
-        config.addAttribute("minLineCount", "2");
-        final String[] expected = {
-            "44:1: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
         verify(config, getPath("InputJavadocMethodExtendAnnotation.java"), expected);
     }
 
-    @Test
-    public void newTest() throws Exception {
-        final DefaultConfiguration config = createModuleConfig(JavadocMethodCheck.class);
-        config.addAttribute("allowedAnnotations", "MyAnnotation, Override");
-        config.addAttribute("minLineCount", "2");
-        final String[] expected = {
-            "57:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
-        verify(config, getPath("InputJavadocMethodSmallMethods.java"), expected);
-    }
-
     @Test
     public void allowedAnnotationsTest() throws Exception {
         final DefaultConfiguration config = createModuleConfig(JavadocMethodCheck.class);
@@ -109,7 +99,6 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         checkConfig.addAttribute("validateThrows", "true");
         final String[] expected = {
-            "14:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
             "18:9: " + getCheckMessage(MSG_UNUSED_TAG, "@param", "unused"),
             "24: " + getCheckMessage(MSG_RETURN_EXPECTED),
             "33: " + getCheckMessage(MSG_RETURN_EXPECTED),
@@ -137,8 +126,9 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
             "262:8: " + getCheckMessage(MSG_UNUSED_TAG, "@param", "aParam"),
             "305: " + getCheckMessage(MSG_RETURN_EXPECTED),
             "305:22: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "aParam"),
-            "328:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "337:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
+            "344:5: " + getCheckMessage(MSG_INVALID_INHERIT_DOC),
+            "383:8: " + getCheckMessage(MSG_DUPLICATE_TAG, "@return"),
+            "389:37: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "algorithm"),
         };
 
         verify(checkConfig, getPath("InputJavadocMethodTags.java"), expected);
@@ -150,7 +140,6 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         checkConfig.addAttribute("allowUndeclaredRTE", "true");
         checkConfig.addAttribute("validateThrows", "true");
         final String[] expected = {
-            "14:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
             "18:9: " + getCheckMessage(MSG_UNUSED_TAG, "@param", "unused"),
             "24: " + getCheckMessage(MSG_RETURN_EXPECTED),
             "33: " + getCheckMessage(MSG_RETURN_EXPECTED),
@@ -175,8 +164,9 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
             "262:8: " + getCheckMessage(MSG_UNUSED_TAG, "@param", "aParam"),
             "305: " + getCheckMessage(MSG_RETURN_EXPECTED),
             "305:22: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "aParam"),
-            "328:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "337:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
+            "344:5: " + getCheckMessage(MSG_INVALID_INHERIT_DOC),
+            "383:8: " + getCheckMessage(MSG_DUPLICATE_TAG, "@return"),
+            "389:37: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "algorithm"),
         };
         verify(checkConfig, getPath("InputJavadocMethodTags.java"), expected);
     }
@@ -185,18 +175,6 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
     public void testStrictJavadoc() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         final String[] expected = {
-            "12:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "18:13: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "25:13: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "38:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "49:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "54:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "59:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "64:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "69:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "74:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "79:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "84:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
             "94:32: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "aA"),
         };
         verify(checkConfig, getPath("InputJavadocMethodPublicOnly.java"), expected);
@@ -215,12 +193,7 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
     public void testRelaxedJavadoc() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         checkConfig.addAttribute("scope", Scope.PROTECTED.getName());
-        final String[] expected = {
-            "59:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "64:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "79:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "84:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
         verify(checkConfig, getPath("InputJavadocMethodPublicOnly.java"), expected);
     }
 
@@ -228,10 +201,7 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
     public void testScopeInnerInterfacesPublic() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         checkConfig.addAttribute("scope", Scope.PUBLIC.getName());
-        final String[] expected = {
-            "43:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "44:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
         verify(checkConfig, getPath("InputJavadocMethodScopeInnerInterfaces.java"), expected);
     }
 
@@ -247,10 +217,7 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
     public void testScopeAnonInnerAnonInner() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         checkConfig.addAttribute("scope", Scope.ANONINNER.getName());
-        final String[] expected = {
-            "26:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "39:17: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "53:17: " + getCheckMessage(MSG_JAVADOC_MISSING), };
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
         verify(checkConfig, getPath("InputJavadocMethodScopeAnonInner.java"), expected);
     }
 
@@ -268,7 +235,6 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         checkConfig.addAttribute("allowThrowsTagsForSubclasses", "true");
         checkConfig.addAttribute("validateThrows", "true");
         final String[] expected = {
-            "14:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
             "18:9: " + getCheckMessage(MSG_UNUSED_TAG, "@param", "unused"),
             "24: " + getCheckMessage(MSG_RETURN_EXPECTED),
             "33: " + getCheckMessage(MSG_RETURN_EXPECTED),
@@ -293,8 +259,9 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
             "262:8: " + getCheckMessage(MSG_UNUSED_TAG, "@param", "aParam"),
             "305: " + getCheckMessage(MSG_RETURN_EXPECTED),
             "305:22: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "aParam"),
-            "328:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "337:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
+            "344:5: " + getCheckMessage(MSG_INVALID_INHERIT_DOC),
+            "383:8: " + getCheckMessage(MSG_DUPLICATE_TAG, "@return"),
+            "389:37: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "algorithm"),
         };
         verify(checkConfig, getPath("InputJavadocMethodTags.java"), expected);
     }
@@ -303,43 +270,10 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
     public void testScopes() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         final String[] expected = {
-            "10:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "11:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "12:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "13:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "21:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "22:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "23:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "24:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "33:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "34:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "35:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "36:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "45:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "46:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "47:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "48:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "58:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "59:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "60:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "61:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "69:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "70:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "71:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "72:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "81:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "82:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "83:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "84:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "93:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "94:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "95:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "96:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "105:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "106:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "107:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "108:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "119:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
+            "15: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+            "17: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+            "19: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+            "21: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
         };
         verify(checkConfig, getPath("InputJavadocMethodNoJavadoc.java"), expected);
     }
@@ -349,10 +283,9 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
         checkConfig.addAttribute("scope", Scope.PROTECTED.getName());
         final String[] expected = {
-            "10:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "11:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "21:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "22:9: " + getCheckMessage(MSG_JAVADOC_MISSING), };
+            "15: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+            "17: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+        };
         verify(checkConfig, getPath("InputJavadocMethodNoJavadoc.java"), expected);
     }
 
@@ -362,43 +295,13 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         checkConfig.addAttribute("scope", Scope.PRIVATE.getName());
         checkConfig.addAttribute("excludeScope", Scope.PROTECTED.getName());
         final String[] expected = {
-            "10:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "12:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "13:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "33:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "35:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "36:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "45:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "47:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "48:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "58:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "60:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "61:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "69:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "71:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "72:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "81:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "83:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "84:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "93:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "95:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "96:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "105:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "107:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "108:9: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "119:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
+            "15: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+            "19: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
+            "21: " + getCheckMessage(MSG_UNUSED_TAG_GENERAL),
         };
         verify(checkConfig, getPath("InputJavadocMethodNoJavadoc.java"), expected);
     }
 
-    @Test
-    public void testAllowMissingJavadoc() throws Exception {
-        final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
-        checkConfig.addAttribute("allowMissingJavadoc", "true");
-        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
-        verify(checkConfig, getPath("InputJavadocMethodNoJavadoc.java"), expected);
-    }
-
     @Test
     public void testAllowMissingJavadocTags() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
@@ -426,48 +329,14 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
     @Test
     public void testSetterGetterOff() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
-        final String[] expected = {
-            "7:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "12:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "17:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "22:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "28:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "32:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "37:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "43:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "48:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "53:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "55:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "59:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "63:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "67:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "69:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "74:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "76:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
         verify(checkConfig, getPath("InputJavadocMethodSetterGetter.java"), expected);
     }
 
     @Test
     public void testSetterGetterOn() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
-        checkConfig.addAttribute("allowMissingPropertyJavadoc", "true");
-        final String[] expected = {
-            "17:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "22:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "28:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "32:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "37:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "43:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "53:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "55:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "59:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "63:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "67:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "69:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "74:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "76:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
         verify(checkConfig, getPath("InputJavadocMethodSetterGetter.java"), expected);
     }
 
@@ -575,26 +444,6 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         verify(checkConfig, getPath("InputJavadocMethodInheritDoc.java"), expected);
     }
 
-    @Test
-    public void testSkipCertainMethods() throws Exception {
-        final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
-        checkConfig.addAttribute("ignoreMethodNamesRegex", "^foo.*$");
-        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
-        verify(checkConfig, getPath("InputJavadocMethodIgnoreNameRegex.java"), expected);
-    }
-
-    @Test
-    public void testNotSkipAnythingWhenSkipRegexDoesNotMatch() throws Exception {
-        final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
-        checkConfig.addAttribute("ignoreMethodNamesRegex", "regexThatDoesNotMatch");
-        final String[] expected = {
-            "5:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "9:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-            "13:5: " + getCheckMessage(MSG_JAVADOC_MISSING),
-        };
-        verify(checkConfig, getPath("InputJavadocMethodIgnoreNameRegex.java"), expected);
-    }
-
     @Test
     public void testMethodsNotSkipWrittenJavadocs() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
@@ -624,4 +473,161 @@ public class JavadocMethodCheckTest extends AbstractModuleTestSupport {
         verify(checkConfig, getPath("InputJavadocMethodReceiverParameter.java"), expected);
     }
 
+    @Test
+    public void testJavadocInMethod() throws Exception {
+        final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
+        verify(checkConfig, getPath("InputJavadocMethodJavadocInMethod.java"), expected);
+    }
+
+    @Test
+    public void testConstructor() throws Exception {
+        final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
+        final String[] expected = {
+            "8:49: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "p1"),
+            "10:50: " + getCheckMessage(MSG_EXPECTED_TAG, "@param", "p1"),
+        };
+        verify(checkConfig, getPath("InputJavadocMethodConstructor.java"), expected);
+    }
+
+    @Test
+    public void testIsSubclassWithNulls() throws Exception {
+        final JavadocMethodCheck check = new JavadocMethodCheck();
+        final Method method = check.getClass()
+                .getDeclaredMethod("isSubclass", Class.class, Class.class);
+        method.setAccessible(true);
+        assertFalse("Should return false if at least one of the params is null",
+            (boolean) method.invoke(check, null, null));
+    }
+
+    @Test
+    public void testGetRequiredTokens() {
+        final int[] expected = {
+            TokenTypes.PACKAGE_DEF,
+            TokenTypes.IMPORT,
+            TokenTypes.CLASS_DEF,
+            TokenTypes.INTERFACE_DEF,
+            TokenTypes.ENUM_DEF,
+        };
+        final JavadocMethodCheck check = new JavadocMethodCheck();
+        final int[] actual = check.getRequiredTokens();
+        assertArrayEquals("Required tokens differ from expected", expected, actual);
+    }
+
+    @Test
+    public void testTokenToString() throws Exception {
+        final Class<?> tokenType = Class.forName("com.puppycrawl.tools.checkstyle.checks.javadoc."
+                + "JavadocMethodCheck$Token");
+        final Constructor<?> tokenConstructor = tokenType.getDeclaredConstructor(String.class,
+                int.class, int.class);
+        final Object token = tokenConstructor.newInstance("blablabla", 1, 1);
+        final Method toString = token.getClass().getDeclaredMethod("toString");
+        final String result = (String) toString.invoke(token);
+        assertEquals("Invalid toString result", "Token[blablabla(1x1)]", result);
+    }
+
+    @Test
+    public void testClassRegularClass() throws Exception {
+        final Class<?> tokenType = Class.forName("com.puppycrawl.tools.checkstyle.checks.javadoc."
+                + "JavadocMethodCheck$Token");
+
+        final Class<?> regularClassType = Class
+                .forName("com.puppycrawl.tools.checkstyle.checks.javadoc."
+                        + "JavadocMethodCheck$RegularClass");
+        final Constructor<?> regularClassConstructor = regularClassType.getDeclaredConstructor(
+                tokenType, String.class, JavadocMethodCheck.class);
+        regularClassConstructor.setAccessible(true);
+
+        try {
+            regularClassConstructor.newInstance(null, "", new JavadocMethodCheck());
+            fail("Exception is expected");
+        }
+        catch (InvocationTargetException ex) {
+            assertTrue("Invalid exception class, expected: IllegalArgumentException.class",
+                ex.getCause() instanceof IllegalArgumentException);
+            assertEquals("Invalid exception message",
+                "ClassInfo's name should be non-null", ex.getCause().getMessage());
+        }
+
+        final Constructor<?> tokenConstructor = tokenType.getDeclaredConstructor(String.class,
+                int.class, int.class);
+        final Object token = tokenConstructor.newInstance("blablabla", 1, 1);
+
+        final JavadocMethodCheck methodCheck = new JavadocMethodCheck();
+        final Object regularClass = regularClassConstructor.newInstance(token, "sur",
+                methodCheck);
+
+        final Method toString = regularClass.getClass().getDeclaredMethod("toString");
+        toString.setAccessible(true);
+        final String result = (String) toString.invoke(regularClass);
+        final String expected = "RegularClass[name=Token[blablabla(1x1)], in class='sur', check="
+                + methodCheck.hashCode() + "," + " loadable=true, class=null]";
+
+        assertEquals("Invalid toString result", expected, result);
+
+        final Method setClazz = regularClass.getClass().getDeclaredMethod("setClazz", Class.class);
+        setClazz.setAccessible(true);
+        final Class<?> arg = null;
+        setClazz.invoke(regularClass, arg);
+
+        final Method getClazz = regularClass.getClass().getDeclaredMethod("getClazz");
+        getClazz.setAccessible(true);
+        assertNull("Expected null", getClazz.invoke(regularClass));
+    }
+
+    @Test
+    public void testClassAliasToString() throws Exception {
+        final Class<?> tokenType = Class.forName("com.puppycrawl.tools.checkstyle.checks.javadoc."
+                + "JavadocMethodCheck$Token");
+
+        final Class<?> regularClassType = Class
+                .forName("com.puppycrawl.tools.checkstyle.checks.javadoc."
+                        + "JavadocMethodCheck$RegularClass");
+        final Constructor<?> regularClassConstructor = regularClassType.getDeclaredConstructor(
+                tokenType, String.class, JavadocMethodCheck.class);
+        regularClassConstructor.setAccessible(true);
+
+        final Constructor<?> tokenConstructor = tokenType.getDeclaredConstructor(String.class,
+                int.class, int.class);
+        final Object token = tokenConstructor.newInstance("blablabla", 1, 1);
+
+        final Object regularClass = regularClassConstructor.newInstance(token, "sur",
+                new JavadocMethodCheck());
+
+        final Class<?> classAliasType = Class.forName(
+                "com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocMethodCheck$ClassAlias");
+        final Class<?> abstractTypeInfoType = Class
+                .forName("com.puppycrawl.tools.checkstyle.checks.javadoc."
+                        + "JavadocMethodCheck$AbstractClassInfo");
+
+        final Constructor<?> classAliasConstructor = classAliasType
+                .getDeclaredConstructor(tokenType, abstractTypeInfoType);
+        classAliasConstructor.setAccessible(true);
+
+        final Object classAlias = classAliasConstructor.newInstance(token, regularClass);
+        final Method toString = classAlias.getClass().getDeclaredMethod("toString");
+        toString.setAccessible(true);
+        final String result = (String) toString.invoke(classAlias);
+        assertEquals("Invalid toString result",
+            "ClassAlias[alias Token[blablabla(1x1)] for Token[blablabla(1x1)]]", result);
+    }
+
+    @Test
+    public void testWithoutLogErrors() throws Exception {
+        final DefaultConfiguration config = createModuleConfig(JavadocMethodCheck.class);
+        config.addAttribute("allowUndeclaredRTE", "true");
+        verify(config, getPath("InputJavadocMethodLoadErrors.java"),
+                CommonUtil.EMPTY_STRING_ARRAY);
+    }
+
+    @Test
+    public void testWithSuppressLoadErrors() throws Exception {
+        final DefaultConfiguration checkConfig = createModuleConfig(JavadocMethodCheck.class);
+        checkConfig.addAttribute("suppressLoadErrors", "true");
+        checkConfig.addAttribute("allowUndeclaredRTE", "true");
+        final String[] expected = CommonUtil.EMPTY_STRING_ARRAY;
+
+        verify(checkConfig, getPath("InputJavadocMethodLoadErrors.java"), expected);
+    }
+
 }
