@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -52,41 +52,45 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
             "16:22: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "="),
-            "16:23: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
-            "18:24: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "16:22: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "18:23: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
             "26:14: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "="),
             "27:10: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "="),
-            "27:11: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "27:10: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
             "28:10: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "+="),
-            "28:12: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "+="),
-            "29:13: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "-="),
-            "37:21: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "synchronized"),
-            "39:12: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "try"),
+            "28:10: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "+="),
+            "29:11: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "-="),
+            "37:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "synchronized"),
+            "39:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "try"),
             "39:12: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
-            "41:14: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "catch"),
+            "41:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "catch"),
             "41:34: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
-            "58:11: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "if"),
-            "76:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "return"),
+            "58:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "if"),
+            "76:13: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "return"),
             "97:29: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "?"),
-            "97:30: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "?"),
+            "97:29: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "?"),
             "97:34: " + getCheckMessage(MSG_WS_NOT_PRECEDED, ":"),
-            "97:35: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, ":"),
+            "97:34: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, ":"),
             "98:15: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "=="),
-            "98:17: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "=="),
-            "104:20: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "*"),
+            "98:15: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "=="),
+            "104:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "*"),
             "104:21: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "*"),
             "119:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "%"),
-            "120:20: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "%"),
+            "120:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "%"),
             "121:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "%"),
-            "121:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "%"),
+            "121:18: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "%"),
             "123:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "/"),
-            "124:20: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "/"),
+            "124:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "/"),
             "125:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "/"),
-            "125:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "/"),
-            "153:15: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "assert"),
+            "125:18: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "/"),
+            "153:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "assert"),
             "156:20: " + getCheckMessage(MSG_WS_NOT_PRECEDED, ":"),
-            "156:21: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, ":"),
-            "262:14: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
+            "156:20: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, ":"),
+            "262:13: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
+            "291:24: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "+"),
+            "291:24: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "+"),
+            "291:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "+"),
+            "291:28: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "+"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundKeywordsAndOperators.java"), expected);
     }
@@ -96,12 +100,12 @@ public class WhitespaceAroundCheckTest
             throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
-            "153:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
-            "154:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
-            "155:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
-            "156:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
-            "157:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
-            "158:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "153:26: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "154:26: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "155:26: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "156:26: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "157:26: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
+            "158:26: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "="),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundSimple.java"), expected);
     }
@@ -121,14 +125,18 @@ public class WhitespaceAroundCheckTest
             throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
-            "37:14: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "while"),
-            "54:12: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "for"),
+            "37:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "while"),
+            "54:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "for"),
             // + ":58:23: ';' is not followed by whitespace.",
             //  + ":58:29: ';' is not followed by whitespace.",
-            "111:43: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "111:42: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "111:43: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "114:40: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "114:39: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "114:40: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
+            "118:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "if"),
+            "118:17: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "118:17: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
+            "118:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundBraces.java"), expected);
     }
@@ -140,8 +148,12 @@ public class WhitespaceAroundCheckTest
         checkConfig.addAttribute("allowEmptyMethods", "true");
         checkConfig.addAttribute("allowEmptyConstructors", "true");
         final String[] expected = {
-            "37:14: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "while"),
-            "54:12: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "for"),
+            "37:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "while"),
+            "54:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "for"),
+            "118:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "if"),
+            "118:17: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "118:17: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
+            "118:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundBraces.java"), expected);
     }
@@ -170,7 +182,7 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
             "7:27: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "&"),
-            "7:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "&"),
+            "7:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "&"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundGenerics.java"), expected);
     }
@@ -188,10 +200,10 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
             "11:73: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "12:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
-            "14:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "12:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "14:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "14:88: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "17:10: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
+            "17:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
             "17:24: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundDoubleBraceInitialization.java"),
@@ -214,11 +226,11 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         checkConfig.addAttribute("allowEmptyTypes", "true");
         final String[] expected = {
-            "29:95: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "29:94: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "29:95: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "30:33: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "30:32: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "30:33: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "31:21: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "31:20: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "31:21: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundEmptyTypesAndCycles.java"),
@@ -230,15 +242,15 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         checkConfig.addAttribute("allowEmptyLoops", "true");
         final String[] expected = {
-            "40:65: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "40:64: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "40:65: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "42:17: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "42:16: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "42:17: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "44:20: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "44:19: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "44:20: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "50:44: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "50:43: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "50:44: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "60:18: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "60:17: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "60:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundEmptyTypesAndCycles.java"),
@@ -249,7 +261,7 @@ public class WhitespaceAroundCheckTest
     public void testSwitchWhitespaceAround() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
-            "6:15: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "switch"),
+            "6:9: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "switch"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundSwitch.java"), expected);
     }
@@ -258,7 +270,7 @@ public class WhitespaceAroundCheckTest
     public void testDoWhileWhitespaceAround() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
-            "9:16: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "while"),
+            "9:11: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "while"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundDoWhile.java"), expected);
     }
@@ -346,15 +358,15 @@ public class WhitespaceAroundCheckTest
             "14:32: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
             "18:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
             "20:24: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
-            "20:25: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "20:24: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "20:31: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "22:31: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
-            "24:18: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "22:30: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
+            "24:17: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "24:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "26:69: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "26:68: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "26:69: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
             "28:24: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
-            "31:13: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "31:12: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "31:13: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundAllowEmptyTypesAndNonEmptyClasses.java"),
@@ -371,11 +383,11 @@ public class WhitespaceAroundCheckTest
             "14:32: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
             "18:18: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
             "20:24: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
-            "20:25: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "20:24: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "20:31: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "22:31: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
+            "22:30: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "}"),
             "28:24: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "{"),
-            "31:13: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "31:12: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "31:13: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
             };
         verify(checkConfig, getPath("InputWhitespaceAroundAllowEmptyTypesAndNonEmptyClasses.java"),
@@ -386,11 +398,11 @@ public class WhitespaceAroundCheckTest
     public void testNotAllowEmptyLambdaExpressionsByDefault() throws Exception {
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
-            "7:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "7:27: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "7:28: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "12:29: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "12:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "12:30: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "13:29: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "13:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "13:42: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundAllowEmptyLambdaExpressions.java"),
@@ -402,9 +414,9 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         checkConfig.addAttribute("allowEmptyLambdas", "true");
         final String[] expected = {
-            "12:29: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "12:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "12:30: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
-            "13:29: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
+            "13:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "{"),
             "13:42: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "}"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundAllowEmptyLambdaExpressions.java"),
@@ -416,7 +428,7 @@ public class WhitespaceAroundCheckTest
         final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
         final String[] expected = {
             "8:48: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "->"),
-            "8:50: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "->"),
+            "8:48: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "->"),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundLambda.java"), expected);
     }
@@ -436,15 +448,43 @@ public class WhitespaceAroundCheckTest
         checkConfig.addAttribute("tokens", "ELLIPSIS");
         final String[] expected = {
             "9:36: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "..."),
-            "10:40: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
+            "10:37: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
             "11:36: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "..."),
-            "11:39: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
+            "11:36: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
             "13:28: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "..."),
-            "13:31: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
+            "13:28: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
             "14:39: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "..."),
-            "14:42: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
+            "14:39: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "..."),
         };
         verify(checkConfig, getPath("InputWhitespaceAroundVarargs.java"), expected);
     }
 
+    @Test
+    public void testWhitespaceAroundAllTokens() throws Exception {
+        final DefaultConfiguration checkConfig = createModuleConfig(WhitespaceAroundCheck.class);
+        checkConfig.addAttribute("tokens", "ASSIGN, ARRAY_INIT, BAND, BAND_ASSIGN, BOR, "
+                + "BOR_ASSIGN, BSR, BSR_ASSIGN, BXOR, BXOR_ASSIGN, COLON, DIV, DIV_ASSIGN, "
+                + "DO_WHILE, EQUAL, GE, GT, LAMBDA, LAND, LCURLY, LE, LITERAL_CATCH, LITERAL_DO, "
+                + "LITERAL_ELSE, LITERAL_FINALLY, LITERAL_FOR, LITERAL_IF, LITERAL_RETURN, "
+                + "LITERAL_SWITCH, LITERAL_SYNCHRONIZED, LITERAL_TRY, LITERAL_WHILE, LOR, LT, "
+                + "MINUS, MINUS_ASSIGN, MOD, MOD_ASSIGN, NOT_EQUAL, PLUS, PLUS_ASSIGN, QUESTION, "
+                + "RCURLY, SL, SLIST, SL_ASSIGN, SR, SR_ASSIGN, STAR, STAR_ASSIGN, "
+                + "LITERAL_ASSERT, TYPE_EXTENSION_AND, WILDCARD_TYPE, GENERIC_START, GENERIC_END, "
+                + "ELLIPSIS");
+        checkConfig.addAttribute("allowEmptyTypes", "true");
+        checkConfig.addAttribute("allowEmptyCatches", "true");
+        final String[] expected = {
+            "6:29: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "<"),
+            "6:29: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "<"),
+            "6:35: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "<"),
+            "6:35: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "<"),
+            "6:36: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, "?"),
+            "6:36: " + getCheckMessage(MSG_WS_NOT_PRECEDED, "?"),
+            "6:37: " + getCheckMessage(MSG_WS_NOT_FOLLOWED, ">"),
+            "6:37: " + getCheckMessage(MSG_WS_NOT_PRECEDED, ">"),
+            "6:38: " + getCheckMessage(MSG_WS_NOT_PRECEDED, ">"),
+        };
+        verify(checkConfig, getPath("InputWhitespaceAroundAllTokens.java"), expected);
+    }
+
 }
