@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 package com.puppycrawl.tools.checkstyle.filters;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -37,10 +38,9 @@ import com.puppycrawl.tools.checkstyle.AbstractModuleTestSupport;
 import com.puppycrawl.tools.checkstyle.DefaultConfiguration;
 import com.puppycrawl.tools.checkstyle.api.AuditEvent;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
+import com.puppycrawl.tools.checkstyle.api.LocalizedMessage;
+import com.puppycrawl.tools.checkstyle.api.SeverityLevel;
 import com.puppycrawl.tools.checkstyle.utils.CommonUtil;
-import nl.jqno.equalsverifier.EqualsVerifier;
-import nl.jqno.equalsverifier.EqualsVerifierReport;
-import nl.jqno.equalsverifier.Warning;
 
 public class SuppressionFilterTest extends AbstractModuleTestSupport {
 
@@ -52,16 +52,6 @@ public class SuppressionFilterTest extends AbstractModuleTestSupport {
         return "com/puppycrawl/tools/checkstyle/filters/suppressionfilter";
     }
 
-    @Test
-    public void testEqualsAndHashCode() {
-        final EqualsVerifierReport ev = EqualsVerifier.forClass(SuppressionFilter.class)
-                .usingGetClass()
-                .withIgnoredFields("file", "optional", "configuration")
-                .suppress(Warning.NONFINAL_FIELDS)
-                .report();
-        assertEquals("Error: " + ev.getMessage(), EqualsVerifierReport.SUCCESS, ev);
-    }
-
     @Test
     public void testAccept() throws Exception {
         final String fileName = getPath("InputSuppressionFilterNone.xml");
@@ -74,6 +64,20 @@ public class SuppressionFilterTest extends AbstractModuleTestSupport {
             filter.accept(ev));
     }
 
+    @Test
+    public void testAcceptFalse() throws Exception {
+        final String fileName = getPath("InputSuppressionFilterSuppress.xml");
+        final boolean optional = false;
+        final SuppressionFilter filter = createSuppressionFilter(fileName, optional);
+
+        final LocalizedMessage message = new LocalizedMessage(1, 1, null, "msg", null,
+                SeverityLevel.ERROR, null, getClass(), null);
+        final AuditEvent ev = new AuditEvent(this, "ATest.java", message);
+
+        assertFalse("Audit event should be rejected when there is a matching suppression",
+            filter.accept(ev));
+    }
+
     @Test
     public void testAcceptOnNullFile() throws CheckstyleException {
         final String fileName = null;
