@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -95,6 +95,16 @@ public class RootNodeTest extends AbstractPathTestSupport {
 
     @Test
     public void testIterate() {
+        assertEquals("Result iterator does not match expected",
+                EmptyIterator.OfNodes.THE_INSTANCE, rootNode.iterateAxis(AxisInfo.FOLLOWING));
+        assertEquals("Result iterator does not match expected",
+                EmptyIterator.OfNodes.THE_INSTANCE,
+                rootNode.iterateAxis(AxisInfo.FOLLOWING_SIBLING));
+        assertEquals("Result iterator does not match expected",
+                EmptyIterator.OfNodes.THE_INSTANCE, rootNode.iterateAxis(AxisInfo.PRECEDING));
+        assertEquals("Result iterator does not match expected",
+                EmptyIterator.OfNodes.THE_INSTANCE,
+                rootNode.iterateAxis(AxisInfo.PRECEDING_SIBLING));
         assertEquals("Result iterator does not match expected",
                 EmptyIterator.OfNodes.THE_INSTANCE, rootNode.iterateAxis(AxisInfo.PARENT));
         assertEquals("Result iterator does not match expected",
