@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2018 the original author or authors.
+// Copyright (C) 2001-2019 the original author or authors.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -69,9 +69,9 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/MODIFIERS",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/MODIFIERS/LITERAL_PUBLIC");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/MODIFIERS",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/MODIFIERS/LITERAL_PUBLIC");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -83,12 +83,12 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']"
-                    + "/OBJBLOCK/METHOD_DEF[@text='callSomeMethod']",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']"
-                    + "/OBJBLOCK/METHOD_DEF[@text='callSomeMethod']/MODIFIERS",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']"
-                    + "/OBJBLOCK/METHOD_DEF[@text='callSomeMethod']/MODIFIERS/LITERAL_PUBLIC");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK"
+                + "/METHOD_DEF[./IDENT[@text='callSomeMethod']]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK"
+                + "/METHOD_DEF[./IDENT[@text='callSomeMethod']]/MODIFIERS",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK"
+                + "/METHOD_DEF[./IDENT[@text='callSomeMethod']]/MODIFIERS/LITERAL_PUBLIC");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -100,18 +100,18 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/SLIST/LITERAL_FOR"
-                    + "/SLIST/VARIABLE_DEF[@text='d']",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/SLIST/LITERAL_FOR"
-                    + "/SLIST/VARIABLE_DEF[@text='d']/MODIFIERS",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/SLIST/LITERAL_FOR"
-                    + "/SLIST/VARIABLE_DEF[@text='d']/TYPE",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/SLIST/LITERAL_FOR"
-                    + "/SLIST/VARIABLE_DEF[@text='d']/TYPE/LITERAL_SHORT");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='callSomeMethod']]/SLIST/LITERAL_FOR/SLIST"
+                + "/VARIABLE_DEF[./IDENT[@text='d']]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='callSomeMethod']]/SLIST/LITERAL_FOR/SLIST"
+                + "/VARIABLE_DEF[./IDENT[@text='d']]/MODIFIERS",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='callSomeMethod']]/SLIST/LITERAL_FOR/SLIST"
+                + "/VARIABLE_DEF[./IDENT[@text='d']]/TYPE",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='callSomeMethod']]/SLIST/LITERAL_FOR/SLIST"
+                + "/VARIABLE_DEF[./IDENT[@text='d']]/TYPE/LITERAL_SHORT");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -123,8 +123,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK/METHOD_DEF[@text='Label']"
-                    + "/SLIST/LITERAL_SWITCH/LCURLY");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='Label']]/SLIST/LITERAL_SWITCH/LCURLY");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -136,8 +136,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK/INSTANCE_INIT"
-                    + "/SLIST/RCURLY");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK"
+                + "/INSTANCE_INIT/SLIST/RCURLY");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -149,10 +149,10 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/VARIABLE_DEF[@text='mUse4']/ASSIGN/EXPR",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/VARIABLE_DEF[@text='mUse4']/ASSIGN/EXPR/DOT");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/VARIABLE_DEF["
+                + "./IDENT[@text='mUse4']]/ASSIGN/EXPR",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/VARIABLE_DEF["
+                + "./IDENT[@text='mUse4']]/ASSIGN/EXPR/DOT");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -164,8 +164,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/LPAREN");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK"
+                + "/METHOD_DEF[./IDENT[@text='callSomeMethod']]/LPAREN");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -199,7 +199,7 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-            "/IMPORT[./DOT[@text='File']]");
+            "/IMPORT[./DOT/IDENT[@text='File']]");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -211,19 +211,19 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='saveUser']/PARAMETERS",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='saveUser']/PARAMETERS/PARAMETER_DEF[@text='name']",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='saveUser']/PARAMETERS/PARAMETER_DEF[@text='name']"
-                    + "/MODIFIERS",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='saveUser']/PARAMETERS/PARAMETER_DEF[@text='name']"
-                    + "/TYPE[@text='String']",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='saveUser']/PARAMETERS/PARAMETER_DEF[@text='name']"
-                    + "/TYPE[@text='String']/IDENT");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='saveUser']]/PARAMETERS",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='saveUser']]/PARAMETERS/PARAMETER_DEF[./IDENT[@text='name']]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='saveUser']]/PARAMETERS/PARAMETER_DEF[./IDENT[@text='name']]"
+                + "/MODIFIERS",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='saveUser']]/PARAMETERS/PARAMETER_DEF[./IDENT[@text='name']]"
+                + "/TYPE[./IDENT[@text='String']]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='saveUser']]/PARAMETERS/PARAMETER_DEF[./IDENT[@text='name']]"
+                + "/TYPE/IDENT[@text='String']");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -235,8 +235,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='Label']/SLIST/LITERAL_SWITCH");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='Label']]/SLIST/LITERAL_SWITCH");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -248,10 +248,10 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK/METHOD_DEF[@text='Label']"
-                    + "/SLIST/LITERAL_SWITCH/CASE_GROUP",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK/METHOD_DEF[@text='Label']"
-                    + "/SLIST/LITERAL_SWITCH/CASE_GROUP/LITERAL_DEFAULT");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='Label']]/SLIST/LITERAL_SWITCH/CASE_GROUP[1]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='Label']]/SLIST/LITERAL_SWITCH/CASE_GROUP/LITERAL_DEFAULT");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -263,12 +263,12 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/SLIST/VARIABLE_DEF[@text='another']"
-                    + "/ASSIGN/EXPR",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='callSomeMethod']/SLIST/VARIABLE_DEF[@text='another']"
-                    + "/ASSIGN/EXPR/STRING_LITERAL");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='callSomeMethod']]/SLIST/VARIABLE_DEF[./IDENT[@text='another']]"
+                + "/ASSIGN/EXPR[./STRING_LITERAL[@text='HelloWorld']]",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='callSomeMethod']]/SLIST/VARIABLE_DEF[./IDENT[@text='another']]"
+                + "/ASSIGN/EXPR/STRING_LITERAL[@text='HelloWorld']");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -280,8 +280,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK/METHOD_DEF[@text='foo']"
-                    + "/SLIST/LITERAL_FOR/FOR_ITERATOR/ELIST/COMMA");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='foo']]/SLIST/LITERAL_FOR/FOR_ITERATOR/ELIST/COMMA");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -293,10 +293,10 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='foo']/TYPE",
-            "/CLASS_DEF[@text='InputXpathQueryGenerator']/OBJBLOCK"
-                    + "/METHOD_DEF[@text='foo']/TYPE/LITERAL_VOID");
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='foo']]/TYPE",
+            "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/OBJBLOCK/METHOD_DEF["
+                + "./IDENT[@text='foo']]/TYPE/LITERAL_VOID");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -308,7 +308,7 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-                "/IMPORT[./DOT[@text='JToolBar']]");
+                "/IMPORT[./DOT/IDENT[@text='JToolBar']]");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -320,7 +320,19 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-                "/IMPORT[./DOT[@text='Iterator']]");
+                "/IMPORT[./DOT/IDENT[@text='Iterator']]");
+        assertEquals("Generated queries do not match expected ones", expected, actual);
+    }
+
+    @Test
+    public void testImportByText() {
+        final int lineNumber = 4;
+        final int columnNumber = 8;
+        final XpathQueryGenerator queryGenerator = new XpathQueryGenerator(rootAst, lineNumber,
+                columnNumber, fileText, DEFAULT_TAB_WIDTH);
+        final List<String> actual = queryGenerator.generate();
+        final List<String> expected = Collections.singletonList(
+                "/IMPORT/DOT[./IDENT[@text='JToolBar']]/DOT/IDENT[@text='javax']");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -332,7 +344,7 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-                "/CLASS_DEF[@text='InputXpathQueryGenerator']/IDENT");
+            "/CLASS_DEF/IDENT[@text='InputXpathQueryGenerator']");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -350,12 +362,12 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, testFileText, tabWidth);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/METHOD_DEF[@text='toString']",
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/METHOD_DEF[@text='toString']/MODIFIERS",
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/METHOD_DEF[@text='toString']/MODIFIERS/LITERAL_PUBLIC");
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                    + "/METHOD_DEF[./IDENT[@text='toString']]",
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                    + "/METHOD_DEF[./IDENT[@text='toString']]/MODIFIERS",
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                    + "/METHOD_DEF[./IDENT[@text='toString']]/MODIFIERS/LITERAL_PUBLIC");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -373,10 +385,10 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, testFileText, tabWidth);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/METHOD_DEF[@text='getName']/TYPE",
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/METHOD_DEF[@text='getName']/TYPE/LITERAL_VOID");
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                    + "/METHOD_DEF[./IDENT[@text='getName']]/TYPE",
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                    + "/METHOD_DEF[./IDENT[@text='getName']]/TYPE/LITERAL_VOID");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -394,8 +406,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, testFileText, tabWidth);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/METHOD_DEF[@text='tabAfterMe']/SLIST");
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                    + "/METHOD_DEF[./IDENT[@text='tabAfterMe']]/SLIST");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -413,8 +425,8 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, testFileText, tabWidth);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-                "/CLASS_DEF[@text='InputXpathQueryGeneratorTabWidth']/OBJBLOCK"
-                        + "/VARIABLE_DEF[@text='endLineTab']/SEMI");
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGeneratorTabWidth']]/OBJBLOCK"
+                        + "/VARIABLE_DEF[./IDENT[@text='endLineTab']]/SEMI");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -426,7 +438,7 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 columnNumber, TokenTypes.CLASS_DEF, fileText, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Collections.singletonList(
-                "/CLASS_DEF[@text='InputXpathQueryGenerator']");
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
@@ -440,9 +452,9 @@ public class XpathQueryGeneratorTest extends AbstractPathTestSupport {
                 new XpathQueryGenerator(event, DEFAULT_TAB_WIDTH);
         final List<String> actual = queryGenerator.generate();
         final List<String> expected = Arrays.asList(
-                "/CLASS_DEF[@text='InputXpathQueryGenerator']",
-                "/CLASS_DEF[@text='InputXpathQueryGenerator']/MODIFIERS",
-                "/CLASS_DEF[@text='InputXpathQueryGenerator']/MODIFIERS/LITERAL_PUBLIC");
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]",
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/MODIFIERS",
+                "/CLASS_DEF[./IDENT[@text='InputXpathQueryGenerator']]/MODIFIERS/LITERAL_PUBLIC");
         assertEquals("Generated queries do not match expected ones", expected, actual);
     }
 
